<?php  
/* 
Plugin Name: Disc Golf Manager
Plugin URI: http://msdiscgolf.com
Description: Disc Golf event management.
Author: Grayson Robbins
Version: 1.0.0 
*/  

class DiscGolfManager {
	public $capability = 'install_plugins';
	public $menu = null;
	public $error = '';
	public $success = '';
	public $flash = array();
	public $name = 'Disc Golf';
	public $key = 'disc_golf';
	public $key_ = 'disc_golf_';
	public $first_time = true;
	public $debug = false;

	function adminBodyClass($classes) {
		global $typenow;

		if($typenow == $this->key_ . 'seasons') {
			return $classes . ' ' . $this->key;
		}
		if($typenow == $this->key_ . 'leagues') {
			return $classes . ' ' . $this->key;
		}
		if($typenow == $this->key_ . 'events') {
			return $classes . ' ' . $this->key;
		}
		if($typenow == $this->key_ . 'players') {
			return $classes . ' ' . $this->key;
		}
		if($typenow == $this->key_ . 'divisions') {
			return $classes . ' ' . $this->key;
		}
		if($typenow == $this->key_ . 'scores') {
			return $classes . ' ' . $this->key;
		}
	}

	function adminEnqueueScripts($hook_suffix) {
		global $post_type;
		$post_types = array(
			$this->key_ . 'seasons',
			$this->key_ . 'leagues',
			$this->key_ . 'events',
			$this->key_ . 'players',
			$this->key_ . 'divisions',
			$this->key_ . 'scores',
		);

		if(in_array($post_type, $post_types)) {
			wp_deregister_script( 'autosave' );  
		}

		if(($hook_suffix == 'post-new.php' || $hook_suffix == 'post.php') && $post_type == $this->key_ . 'scores' ) {
			$file = 'js/admin_scores.js';
			if($this->debug) {
				$file .= '?no_cache=' . rand();
			}
			wp_enqueue_script($this->key_ . 'scores' . '_main', plugins_url($file, __FILE__), array('jquery'));
			$file = 'js/select2.min.js';
			wp_enqueue_script($this->key_ . 'scores' . '_select_js', plugins_url($file, __FILE__), array('jquery'));
			$file = 'css/select2.min.css';
			wp_enqueue_style($this->key_ . 'scores' . '_select_css', plugins_url($file, __FILE__));
		}

		if(($hook_suffix == 'post-new.php' || $hook_suffix == 'post.php') && $post_type == $this->key_ . 'leagues' ) {
			$file = 'js/admin_leagues.js';
			if($this->debug) {
				$file .= '?no_cache=' . rand();
			}
			wp_enqueue_script($this->key_ . 'scores' . '_main', plugins_url($file, __FILE__), array('jquery'));
		}

		if(($hook_suffix == 'post-new.php' || $hook_suffix == 'post.php') && $post_type == $this->key_ . 'players' ) {
			$file = 'js/admin_players.js';
			if($this->debug) {
				$file .= '?no_cache=' . rand();
			}
			wp_enqueue_script($this->key_ . 'scores' . '_main', plugins_url($file, __FILE__), array('jquery'));
		}

		if(($hook_suffix == 'post-new.php' || $hook_suffix == 'post.php') && $post_type == $this->key_ . 'events' ) {
			$file = 'js/admin_events.js';
			if($this->debug) {
				$file .= '?no_cache=' . rand();
			}
			wp_enqueue_script($this->key_ . 'scores' . '_main', plugins_url($file, __FILE__), array('jquery'));
		}

		if(($hook_suffix == 'post-new.php' || $hook_suffix == 'post.php') && $post_type == $this->key_ . 'seasons' ) {
			$file = 'js/admin_seasons.js';
			if($this->debug) {
				$file .= '?no_cache=' . rand();
			}
			wp_enqueue_script($this->key_ . 'scores' . '_main', plugins_url($file, __FILE__), array('jquery'));
		}

		//$file = 'css/jquery-ui.css';
		//wp_enqueue_style('jquery-ui', plugins_url($file, __FILE__));
		wp_register_style('jquery-ui', 'http://ajax.googleapis.com/ajax/libs/jqueryui/1.8/themes/base/jquery-ui.css');
		wp_enqueue_style( 'jquery-ui' );

		$file = 'js/admin.js';
		if($this->debug) {
			$file .= '?no_cache=' . rand();
		}
		wp_enqueue_script($this->key . '_main', plugins_url($file, __FILE__), array('jquery', 'jquery-ui-core', 'jquery-ui-datepicker'));
		$file = 'css/admin.css';
		if($this->debug) {
			$file .= '?no_cache=' . rand();
		}
		wp_enqueue_style($this->key . '_main', plugins_url($file, __FILE__));
	}

	function adminInit() {
		global $typenow;

		if(
			$typenow == $this->key_ . 'scores'
			|| $typenow == $this->key_ . 'seasons'
			|| $typenow == $this->key_ . 'leagues'
			|| $typenow == $this->key_ . 'events'
			|| $typenow == $this->key_ . 'players'
			|| $typenow == $this->key_ . 'divisions'
		) {
			add_filter('months_dropdown_results', '__return_empty_array');
		}
	}

	function adminMenu() {
		add_menu_page($this->name, $this->name, $this->capability, $this->key, false);

		add_submenu_page($this->parseToKey('Disc Golf'), 'Import', 'Import', $this->key_ . 'edit_tournaments', $this->key_ . 'import', array($this, 'import'));
		add_submenu_page($this->parseToKey('Disc Golf'), 'Settings', 'Settings', 'install_plugins', $this->key_ . 'settings', array($this, 'settings'));

		// These pages have to be added (but hidden) to menu so that tournament director can access them
		add_submenu_page('edit.php?post_type=' . $this->key_ . 'seasons', 'Hidden', 'Hidden', $this->key_ . 'edit_tournaments', 'post-new.php?post_type=' . $this->key_ . 'seasons', false);
		add_submenu_page('edit.php?post_type=' . $this->key_ . 'leagues', 'Hidden', 'Hidden', $this->key_ . 'edit_tournaments', 'post-new.php?post_type=' . $this->key_ . 'leagues', false);
		add_submenu_page('edit.php?post_type=' . $this->key_ . 'events', 'Hidden', 'Hidden', $this->key_ . 'edit_tournaments', 'post-new.php?post_type=' . $this->key_ . 'events', false);
		add_submenu_page('edit.php?post_type=' . $this->key_ . 'players', 'Hidden', 'Hidden', $this->key_ . 'edit_tournaments', 'post-new.php?post_type=' . $this->key_ . 'players', false);
		add_submenu_page('edit.php?post_type=' . $this->key_ . 'scores', 'Hidden', 'Hidden', $this->key_ . 'edit_tournaments', 'post-new.php?post_type=' . $this->key_ . 'scores', false);
	}

	function actionAdminImport() {
		$current_user = wp_get_current_user();
		$clear = false;
		$message = '';

		$message .= '|' . $this->val('_wpnonce', 'import', 'nonce');
		$message .= '|' . $this->val('event', 'Event', 'required');

		if(empty($_FILES['file']['name'])) {
			$message .= '|' . 'File is required.';
		}

		if($_FILES['file']['name'] && !in_array(strtolower(end(explode('.', $_FILES['file']['name']))), array('xls', 'xlsx'))){
			$message .= '|' . 'File is not valid.';
		}


		// Check if event
		if(isset($_POST['event']) && is_numeric($_POST['event'])) {
			if(in_array($this->key_ . 'tournament_director', $current_user->roles)) {
				$post = get_post( sanitize_text_field($_POST['event']) );

				if($post->post_author != $current_user->ID) {
					$message .= '|' . 'There was a problem processing the request.';
				}

			}
			if(isset($_POST['clear']) && $_POST['clear'] == 'yes') {
				$clear = true;
			}
		}

		$message = $this->parseMessage($message, "\n");

		if(!$message) {
			$event_id = sanitize_text_field($_POST['event']);

			// Process
			include 'includes' . DIRECTORY_SEPARATOR . 'simplexlsx.class.php';

			$xlsx = new SimpleXLSX($_FILES['file']['tmp_name']);
			if($xlsx->success()){
				$i = 0;
				$scores = array();
				foreach($xlsx->rows() as $index => $c) {
					if($i == 0) {
					} elseif($c[0]) {
						$temp = array();
						$temp['event_id'] = $event_id;
						$temp['division_name'] = $c[0];
						$temp['first_name'] = $c[1];
						$temp['last_name'] = $c[2];
						$temp['email'] = $c[3];
						$temp['pdga'] = $c[5];
						$temp['r1'] = $c[6] - 0;
						$temp['r2'] = $c[7] - 0;
						$temp['r3'] = $c[8] - 0;
						$temp['r4'] = $c[9] - 0;
						$temp['r5'] = $c[10] - 0;
						if($c[11] - 0 == 1) {
							$temp['playoff_winner'] = 'Yes';
						} else {
							$temp['playoff_winner'] = 'No';
						}
						$temp['winnings'] = $c[12] - 0;
						$fn_ln_email = '';
						$fn_ln_email .= preg_replace('/[^a-z,0-9]/', '', strtolower($temp['first_name']));
						$fn_ln_email .= '_';
						$fn_ln_email .= preg_replace('/[^a-z,0-9]/', '', strtolower( $temp['last_name']));
						$fn_ln_email .= '_';
						$fn_ln_email .= strtolower($temp['email']);
						$temp['fn_ln_email'] = $fn_ln_email;
						array_push($scores, $temp);

					}
					$i++;
				}

				remove_action('save_post_' . $this->key_ . 'players', array($this, 'playersSave'));
				remove_action('save_post_' . $this->key_ . 'scores', array($this, 'scoresSave'));

				// Get divisions
				$divisions = get_posts([
					'post_type' => $this->key_ . 'divisions',
					'post_status' => 'publish',
					'numberposts' => -1,
				]);
				for($i = 0; $i < count($scores); $i++) {
					for($j = 0; $j < count($divisions); $j++) {
						if($divisions[$j]->post_title == $scores[$i]['division_name']) {
							$scores[$i]['division_id'] = $divisions[$j]->ID;
							break;
						}
					}
					if(!isset($scores[$i]['division_id'])) {
						$scores[$i]['division_id'] = -1;
					}
				}

				// Get players
				$players = array();
				$posts = get_posts([
					'post_type' => $this->key_ . 'players',
					'post_status' => 'publish',
					'numberposts' => -1,
					'fields' => 'ids',
				]);

				for($j = 0; $j < count($posts); $j++) {
					$temp = array();
					$temp['id'] = $posts[$j];
					$temp['pdga'] = get_post_meta( $posts[$j], 'pdga', true );
					$temp['fn_ln_email'] = get_post_meta( $posts[$j], 'fn_ln_email', true );
					array_push($players, $temp);
				}

				for($i = 0; $i < count($scores); $i++) {
					for($j = 0; $j < count($players); $j++) {
						if($scores[$i]['pdga'] && $players[$j]['pdga'] == $scores[$i]['pdga']) {
							$scores[$i]['player_id'] = $players[$j]['id'];
							break;
						}
						if($players[$j]['fn_ln_email'] == $scores[$i]['fn_ln_email']) {
							$scores[$i]['player_id'] = $players[$j]['id'];
							break;
						}
					}
					if(!isset($scores[$i]['player_id'])) {
						$scores[$i]['player_id'] = -1;
					}
				}

				// Create new players
				for($i = 0; $i < count($scores); $i++) {
					if($scores[$i]['player_id'] == -1) {
						$name = $scores[$i]['first_name'];
						$name .= ' ';
						$name .= $scores[$i]['last_name'];
						$player_id = wp_insert_post(array (
							'post_type' => $this->key_ . 'players',
							'post_title' => $name,
							'post_content' => '',
							'post_status' => 'publish'
						));

						update_post_meta($player_id, 'first_name', $scores[$i]['first_name']);
						update_post_meta($player_id, 'last_name', $scores[$i]['last_name']);
						update_post_meta($player_id, 'email', $scores[$i]['email']);
						update_post_meta($player_id, 'pdga', $scores[$i]['pdga']);
						update_post_meta($player_id, 'fn_ln_email', $scores[$i]['fn_ln_email']);

						$scores[$i]['player_id'] = $player_id;
					}
				}

				// Clear old scores
				if($clear) {
					$posts = get_posts([
						'post_type' => $this->key_ . 'scores',
						'post_status' => 'publish',
						'numberposts' => -1,
						'fields' => 'ids',
						'meta_key' => 'event',
						'meta_value' => $event_id,
					]);
					foreach($posts as $post_id) {
						wp_delete_post($post_id);
					}
				}

				// Create score
				for($i = 0; $i < count($scores); $i++) {
					if($scores[$i]['division_id'] != -1) {
						$score_id = wp_insert_post(array (
							'post_type' => $this->key_ . 'scores',
							'post_title' => 'Excel Insert',
							'post_content' => '',
							'post_status' => 'publish'
						));

						$total = 0;
						$total += $scores[$i]['r1'] - 0;
						$total += $scores[$i]['r2'] - 0;
						$total += $scores[$i]['r3'] - 0;
						$total += $scores[$i]['r4'] - 0;
						$total += $scores[$i]['r5'] - 0;
						update_post_meta($score_id, 'event', $scores[$i]['event_id']);
						update_post_meta($score_id, 'division', $scores[$i]['division_id']);
						update_post_meta($score_id, 'player', $scores[$i]['player_id']);
						update_post_meta($score_id, 'r1', $scores[$i]['r1']);
						update_post_meta($score_id, 'r2', $scores[$i]['r2']);
						update_post_meta($score_id, 'r3', $scores[$i]['r3']);
						update_post_meta($score_id, 'r4', $scores[$i]['r4']);
						update_post_meta($score_id, 'r5', $scores[$i]['r5']);
						update_post_meta($score_id, 'total', $total);
						update_post_meta($score_id, 'winnings', $scores[$i]['winnings']);
						update_post_meta($score_id, 'playoff_winner', $scores[$i]['playoff_winner']);
					}
				}

				add_action('save_post_' . $this->key_ . 'players', array($this, 'playersSave'));
				add_action('save_post_' . $this->key_ . 'scores', array($this, 'scoresSave'));

				// Update Points
				$this->updateEventPointsRank($event_id);

				$this->flashSave('Excel file has been processed.', 'success');
			} else {
				$this->flashSave('There was a problem reading the excel file.', 'error');
			}
		
			wp_redirect('admin.php?page=disc_golf_import');
			exit;
		} else {
			//$output = array();
			//$output['status'] = 'error';
			//$output['message'] = $message;
			//http_response_code(400);
			//echo json_encode($output);

			$this->flashSave($message, 'error');

			wp_redirect('admin.php?page=disc_golf_import');
			exit;
		}
		die;
	}

	function actionAdminSettings() {
		$current_user = wp_get_current_user();
		$clear = false;
		$message = '';

		$message .= '|' . $this->val('_wpnonce', 'settings', 'nonce');
		$message .= '|' . $this->val('event', 'Event Page', 'required');
		$message .= '|' . $this->val('league', 'League Page', 'required');
		$message .= '|' . $this->val('player', 'Player Page', 'required');

		$message = $this->parseMessage($message, "\n");

		if(!$message) {
			$event_id = sanitize_text_field($_POST['event']);
			$league_id = sanitize_text_field($_POST['league']);
			$player_id = sanitize_text_field($_POST['player']);

			update_option($this->key_ . 'event_page', $event_id);
			update_option($this->key_ . 'league_page', $league_id);
			update_option($this->key_ . 'player_page', $player_id);

			wp_redirect('admin.php?page=disc_golf_settings');
			exit;
		} else {

			$this->flashSave($message, 'error');

			wp_redirect('admin.php?page=disc_golf_settings');
			exit;
		}
		die;
	}

	function activation() {
		global $wpdb;
		$role = add_role(
			$this->key_ . 'tournament_director', 
			__('Tournament Director'),
			array(
				'read' => true
			)
		);

		$role = get_role($this->key_ . 'tournament_director');
		$role->add_cap($this->key_ . 'edit_tournaments');

		$role = get_role('administrator');
		$role->add_cap($this->key_ . 'edit_tournaments');

		$role = get_role('editor');
		$role->add_cap($this->key_ . 'edit_tournaments');

		$table_name = $wpdb->prefix . "posts";
		$query = $wpdb->prepare("
			SELECT *
			FROM " . $table_name . "
			WHERE post_type = %s
			AND post_status = %s
			", $this->key_ .'divisions', 'publish');
		$divisions = $wpdb->get_results($query, ARRAY_A);         

		if(count($divisions) == 0) {
			$default_divisions = array();
			array_push($default_divisions, 'Pro - Men');
			array_push($default_divisions, 'Pro Master - Men');
			array_push($default_divisions, 'Pro Grandmaster - Men');
			array_push($default_divisions, 'Am - Men');
			array_push($default_divisions, 'Advanced - Men');
			array_push($default_divisions, 'Advanced Master - Men');
			array_push($default_divisions, 'Advanced Grandmaster - Men');
			array_push($default_divisions, 'Pro - Women');
			array_push($default_divisions, 'Am - Women');
			array_push($default_divisions, 'Advanced - Women');
			array_push($default_divisions, 'Junior Girl');
			array_push($default_divisions, 'Junior Boy ');
			array_push($default_divisions, 'Recreational');
			array_push($default_divisions, 'Novice');
			array_push($default_divisions, 'Doubles');
			array_push($default_divisions, 'Mixed Doubles');
			array_push($default_divisions, 'DD - A Class');
			array_push($default_divisions, 'Pro Senior Grandmaster - Men ');
			array_push($default_divisions, 'Pro Master - Women');

			for($i = 0; $i < count($default_divisions); $i++) {
				$post_id = wp_insert_post(array (
					'post_type' => $this->key_ . 'divisions',
					'post_title' => $default_divisions[$i],
					'post_content' => '',
					'post_status' => 'publish'
				));
			}
		}
	}

	function deactivation() {
		remove_role($this->key_ . 'tournament_director');

		$role = get_role('administrator');
		$role->remove_cap($this->key_ . 'edit_tournaments');

		$role = get_role('editor');
		$role->remove_cap($this->key_ . 'edit_tournaments');
	}

	function divisionsColumns() {
		$columns = array(
			'cb' => '<input type="checkbox" />',
			'title' => __( 'Name' ),
		);

		return $columns;
	}

	function divisionsColumnContent($column, $post_id) {
		if($column == 'name') {
			$name = get_the_title($post_id);
			$title = '<a class="row-title" href="post.php?post=' . $post_id . '&action=edit">' . $name . '</a>';
			echo $title;
		}
	}

	function divisionsMeta() {
		remove_meta_box( 'submitdiv', $this->key_ . 'divisions', 'side' );
		add_meta_box(
			$this->key_ . 'divisions' . '_save',
			'Save',
			array($this, 'metaSave'),
			$this->key_ . 'divisions',
			'side',
			'high'
		);
	}

	function doNothing() {
	}

	function eventsColumns() {
		$columns = array(
			'cb' => '<input type="checkbox" />',
			'title' => __( 'Title' ),
			'tournament_director' => __( 'Tournament Director' ),
			'start_date' => __( 'Start Date' ),
			'end_date' => __( 'End Date' ),
			'par' => __( 'Par' ),
		);

		return $columns;
	}

	function eventsColumnContent($column, $post_id) {
		if($column == 'title_only') {
			$title = '<a class="row-title" href="post.php?post=' . $post_id . '&action=edit">' . get_the_title( $post_id) . '</a>';
			echo $title;
		} elseif($column == 'tournament_director') {
			$user = get_user_by('id', get_post_meta( $post_id, 'tournament_director', true ));
			if($user) {
				echo $user->display_name;
			}
		} elseif($column == 'start_date') {
			echo get_post_meta( $post_id, 'start_date', true );
		} elseif($column == 'end_date') {
			echo get_post_meta( $post_id, 'end_date', true );
		} elseif($column == 'par') {
			echo get_post_meta( $post_id, 'par', true );
		}
	}

	function eventsColumnSort($cols) {
		return array(
			'title' => 'title',
			'start_date' => 'start_date'
		);
	}

	function eventsMeta() {
		add_meta_box(
			$this->key_ . 'events_details',
			'Event Details',
			array($this, 'eventsDetails'),
			$this->key_ . 'events',
			'normal',
			'default'
		);
		add_meta_box(
			$this->key_ . 'events_location',
			'Location Details',
			array($this, 'eventsLocation'),
			$this->key_ . 'events',
			'normal',
			'default'
		);
		remove_meta_box( 'submitdiv', $this->key_ . 'events', 'side' );
		add_meta_box(
			$this->key_ . 'events_save',
			'Save',
			array($this, 'metaSave'),
			$this->key_ . 'events',
			'side',
			'high'
		);
		/*
		add_meta_box(
			$this->key_ . 'events_comments',
			'Comments',
			array($this, 'eventsComments'),
			$this->key_ . 'events',
			'side',
			'default'
		);
		 */
	}

	function eventsComments() {
		global $post;

		$data = array();

		$data['comments'] = get_post_meta( $post->ID, 'comments', true );

		echo $this->show('meta_box_events_comments', $data);
	}

	function eventsDetails() {
		global $post;
		$current_user = wp_get_current_user();
		$editing = true;

		if (!in_array( $post->post_status, array('publish', 'future', 'private') ) || 0 == $post->ID ) {
			// Initial save
			$editing = false;
		}

		$data = array();

		// Get current seasons
		$season_ids = $this->shortGetActiveSeasons();

		$league_ids = get_posts([
			'post_type' => $this->key_ . 'leagues',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
			'meta_query' => array(
				array(
					'key' => 'season',
					'value' => $season_ids,
					'compare' => 'IN'
				)
			)
		]);

		if($editing) {
			// show all
			$league_ids = array();
		}

		$data['leagues'] = wp_dropdown_pages(array(
			'show_option_none' => __( 'Please Select...' ),
			'post_type'=> $this->key_ . 'leagues',
			'name' => 'league',
			'echo' => 0,
			'selected' => get_post_meta( $post->ID, 'league', true ),
			'include' => implode(',', $league_ids)
		));

		if(in_array($this->key_ . 'tournament_director', $current_user->roles)) {
			$data['show_td'] = false;
			$data['td'] = '<input type="hidden" name="tournament_director" value="' . $current_user->ID . '" />';
		} else {
			$data['show_td'] = true;
			$data['td'] = wp_dropdown_users(array(
				'show_option_none' => __( 'Please Select...' ),
				//'role'=> $this->key_ . 'tournament_director',
				'role__in' => array('administrator', $this->key_ . 'tournament_director'),
				'name' => 'tournament_director',
				'echo' => 0,
				'selected' => get_post_meta( $post->ID, 'tournament_director', true )
			));
		}

		$data['doubles'] = get_post_meta( $post->ID, 'doubles', true );

		$data['comments'] = get_post_meta( $post->ID, 'comments', true );

		echo $this->show('meta_box_events_details', $data);
	}

	function eventsLocation() {
		global $post;

		$data = array();
		$data['start_date'] = get_post_meta( $post->ID, 'start_date', true );
		$data['end_date'] = get_post_meta( $post->ID, 'end_date', true );
		$data['par'] = get_post_meta( $post->ID, 'par', true );
		$data['course_name'] = get_post_meta( $post->ID, 'course_name', true);
		$data['city'] = get_post_meta( $post->ID, 'city', true);
		$data['state'] = get_post_meta( $post->ID, 'state', true);
		$data['website'] = get_post_meta( $post->ID, 'website', true);

		echo $this->show('meta_box_events_location', $data);
	}

	function eventsSave($post_id) {
		if(!empty($_POST)) {
			$last_start_date = get_post_meta( $post_id, 'start_date', true );

			update_post_meta($post_id, 'league', sanitize_text_field( $_POST['league']));
			if(isset($_POST['doubles'])) {
				update_post_meta($post_id, 'doubles', sanitize_text_field( $_POST['doubles']));
			}
			update_post_meta($post_id, 'tournament_director', sanitize_text_field( $_POST['tournament_director']));

			update_post_meta($post_id, 'start_date', sanitize_text_field( $_POST['start_date']));
			update_post_meta($post_id, 'end_date', sanitize_text_field( $_POST['end_date']));
			update_post_meta($post_id, 'par', sanitize_text_field( $_POST['par']));
			update_post_meta($post_id, 'course_name', sanitize_text_field( $_POST['course_name']));
			update_post_meta($post_id, 'city', sanitize_text_field( $_POST['city']));
			update_post_meta($post_id, 'state', sanitize_text_field( $_POST['state']));
			update_post_meta($post_id, 'website', sanitize_text_field( $_POST['website']));

			update_post_meta($post_id, 'comments', wp_filter_post_kses(  $_POST['comments']));

			// Deactivate save_post
			remove_action('save_post_' . $this->key_ . 'events', array($this, 'eventsSave'));
			remove_action('save_post_' . $this->key_ . 'scores', array($this, 'scoresSave'));
			if(isset($_POST['tournament_director']) && is_numeric($_POST['tournament_director'])) {

				$args = array('ID' => $post_id, 'post_author' => sanitize_text_field( $_POST['tournament_director']));
				wp_update_post( $args );

				// Update each score associated with event
				$posts = get_posts([
					'post_type' => $this->key_ . 'scores',
					'post_status' => 'publish',
					'numberposts' => -1,
					'fields' => 'ids',
					'meta_key' => 'event',
					'meta_value' => $post_id
				]);

				foreach($posts as $temp_id) {
					$args = array('ID' => $temp_id, 'post_author' => sanitize_text_field( $_POST['tournament_director']));
					wp_update_post( $args );
				}

			} else {
				// Update each score associated with event
				$posts = get_posts([
					'post_type' => $this->key_ . 'scores',
					'post_status' => 'publish',
					'numberposts' => -1,
					'fields' => 'ids',
					'meta_key' => 'event',
					'meta_value' => $post_id
				]);

				foreach($posts as $temp_id) {
					$args = array('ID' => $temp_id, 'post_author' => get_current_user_id());
					wp_update_post( $args );
				}
			}
			// Reactivate save_post
			add_action('save_post_' . $this->key_ . 'events', array($this, 'eventsSave'));
			add_action('save_post_' . $this->key_ . 'scores', array($this, 'scoresSave'));

			$new_start_date = sanitize_text_field( $_POST['start_date']);
			if($last_start_date != $new_start_date) {
				$this->updateEventPoints($post_id, $last_start_date, $new_start_date);
			}
		}
	}

	function filterPosts($post_id) {
		global $typenow;

		if($typenow == $this->key_ . 'scores') {
			if(isset($_GET['filter_event'])) {
				$selected = $_GET['filter_event'];
			} else {
				$selected = '';
			}

			wp_dropdown_pages(array(
				'show_option_none' => __( 'All Events' ),
				'post_type'=> $this->key_ . 'events',
				'name' => 'filter_event',
				'echo' => 1,
				'selected' => $selected
			));
		}
	}

	function highlightMenu( $parent_file ) {
		global $submenu_file, $current_screen, $pagenow;

		# Set the submenu as active/current while anywhere in your Custom Post Type
		if ( $current_screen->post_type == $this->key_ . 'leagues' ) {
			if( $pagenow == 'edit-tags.php' ) {
				$submenu_file = 'edit-tags.php?taxonomy=' . $this->key_ . 'leagues' . '&post_type=' . $current_screen->post_type;
			}
			if( $pagenow == 'post-new.php' ) {
				$submenu_file = 'post-new.php?post_type=disc_golf_leagues';
				$submenu_file = 'edit.php?post_type=disc_golf_leagues';
			}

			//$submenu_file = 'edit.php?post_type=disc_golf_leagues';

			$parent_file = $this->key;
		}

		return $parent_file;

	}
	function highlightMenu2( $submenu_file, $parent_file ) {

		//$submenu_file = 'edit.php?post_type=disc_golf_leagues';
		//echo $submenu_file;
		//echo '<br>';
		//echo $parent_file;
		//die;
		return $submenu_file;
	}

	function highlightSeasonMenu( $parent_file ) {
		global $submenu_file, $current_screen, $pagenow;

		# Set the submenu as active/current while anywhere in your Custom Post Type
		if ( $current_screen->post_type == $this->key_ . 'leagues' ) {
			if( $pagenow == 'edit-tags.php' ) {
				$submenu_file = 'edit-tags.php?taxonomy=' . $this->key_ . 'seasons' . '&post_type=' . $current_screen->post_type;
			}

			$parent_file = $this->key;
		}

		return $parent_file;

	}

	function import() {
		$data = array();
		$current_user = wp_get_current_user();

		if(in_array($this->key_ . 'tournament_director', $current_user->roles)) {
			$posts = get_posts([
				'author' => $current_user->ID,
				'post_type' => $this->key_ . 'events',
				'post_status' => 'publish',
				'numberposts' => -1,
				'fields' => 'ids'
			]);
			$data['events'] = wp_dropdown_pages(array(
				'show_option_none' => __( 'Please Select...' ),
				'post_type'=> $this->key_ . 'events',
				'name' => 'event',
				'echo' => 0,
				'include' => implode(',', $posts),
			));
		} else {
			$data['events'] = wp_dropdown_pages(array(
				'show_option_none' => __( 'Please Select...' ),
				'post_type'=> $this->key_ . 'events',
				'name' => 'event',
				'echo' => 0,
			));
		}
		echo $this->show('admin_import', $data);
	}

	function leagueColumns() {
		$columns = array(
			'cb' => '<input type="checkbox" />',
			'title' => __( 'Title' ),
			'season' => __( 'Season' )
		);

		return $columns;
	}

	function leagueColumnContent($column, $post_id) {
		if($column == 'title_only') {
			$title = '<a class="row-title" href="post.php?post=' . $post_id . '&action=edit">' . get_the_title( $post_id) . '</a>';
			echo $title;
		} else if($column == 'season') {
			$season = get_the_title(get_post_meta( $post_id, 'season', true ));
			echo $season;
		}
	}

	function leaguesMeta() {
		add_meta_box(
			$this->key_ . 'leagues_season',
			'Season',
			array($this, 'leaguesSeason'),
			$this->key_ . 'leagues',
			'side',
			'default'
		);
		remove_meta_box( 'submitdiv', $this->key_ . 'leagues', 'side' );
		add_meta_box(
			$this->key_ . 'leagues' . '_save',
			'Save',
			array($this, 'metaSave'),
			$this->key_ . 'leagues',
			'side',
			'high'
		);
	}

	function leaguesSeason() {
		global $post;

		$editing = true;
		$season_id = get_post_meta( $post->ID, 'season', true );
		$season_name = get_the_title($season_id);

		$data = array();
		$data['seasons'] = wp_dropdown_pages(array(
			'show_option_none' => __( 'Please Select...' ),
			'post_type'=> $this->key_ . 'seasons',
			'name' => 'season',
			'echo' => 0,
			'selected' => $season_id
		));

		// From: https://gist.github.com/NiloySarker/2d1954eef3b0003d718d#file-replace-wp_submit-php-L93
		if (!in_array( $post->post_status, array('publish', 'future', 'private') ) || 0 == $post->ID ) {
			// Initial save
			$editing = false;
		}

		if(!current_user_can('administrator') && $editing) {
			$data['seasons'] = $season_name . '<input type="hidden" name="season" value="' . $season_id . '" />';
		}

		echo $this->show('meta_box_leagues_seasons', $data);
	}

	function leaguesSave($post_id) {
		if(!empty($_POST)) {
			update_post_meta($post_id, 'season', sanitize_text_field( $_POST['season']));
		}
	}

	// Inspired by: http://justintadlock.com/archives/2010/07/10/meta-capabilities-for-custom-post-types
	function mapMetaCap($caps, $cap, $user_id, $args) {
		if($cap == $this->key_ . 'edit_tourn') {
			$post = get_post( $args[0] );
			$post_type = get_post_type_object( $post->post_type );

			$caps = array();
			if($post->post_author == $user_id) {
				$caps[] = $post_type->cap->edit_posts;
			} else {
				$caps[] = $post_type->cap->edit_others_posts;
			}
		}
		return $caps;
	}

	function metaSave() {
		global $post;

		$data = array();
		// From: https://gist.github.com/NiloySarker/2d1954eef3b0003d718d#file-replace-wp_submit-php-L93
		if (!in_array( $post->post_status, array('publish', 'future', 'private') ) || 0 == $post->ID ) {
			$data['initial_save'] = true;
		} else {
			$data['initial_save'] = false;
		}

		$data['redirect_info'] = array();
		$valid_redirects = array('score', 'score2', 'score-edit', 'score2-edit');
		if(isset($_GET['redirect']) && in_array($_GET['redirect'], $valid_redirects)) {
			$data['redirect_info']['redirect'] = $_GET['redirect'];
		}
		if(isset($_GET['post_id']) && is_numeric($_GET['post_id'])) {
			$data['redirect_info']['post_id'] = $_GET['post_id'];
		}
		if(isset($_GET['event']) && is_numeric($_GET['event'])) {
			$data['redirect_info']['event'] = $_GET['event'];
		}
		if(isset($_GET['division']) && is_numeric($_GET['division'])) {
			$data['redirect_info']['division'] = $_GET['division'];
		}
		if(isset($_GET['player']) && is_numeric($_GET['player'])) {
			$data['redirect_info']['player'] = $_GET['player'];
		}
		if(isset($_GET['player2']) && is_numeric($_GET['player2'])) {
			$data['redirect_info']['player2'] = $_GET['player2'];
		}

		echo $this->show('meta_box_events_save', $data);
	}

	function parsePosts($query) {
		global $pagenow;
		$post_type = isset( $_GET['post_type'] ) ? $_GET['post_type'] : '';
		$filter_event = isset( $_GET['filter_event'] ) ? $_GET['filter_event'] : '';
		if(is_admin() && $pagenow=='edit.php' && $post_type == $this->key_ . 'scores' && $filter_event) {
			$query->query_vars['meta_key'] = 'event';
			$query->query_vars['meta_value'] = $filter_event;
			$query->query_vars['meta_compare'] = '=';
		}
	}

	function playersColumns() {
		$columns = array(
			'cb' => '<input type="checkbox" />',
			'title' => __( 'Name' ),
			'email' => __( 'Email' ),
		);

		return $columns;
	}

	function playersColumnContent($column, $post_id) {
		if($column == 'name') {
			$first = get_post_meta( $post_id, 'first_name', true );
			$last = get_post_meta( $post_id, 'last_name', true );
			$name = $first . ' ' . $last;
			if($name == ' ') {
				$name = '(no name)';
				$title = '<a class="row-title" href="post.php?post=' . $post_id . '&action=edit">' . $name . '</a>';
			} else {
				$title = '<a class="row-title" href="post.php?post=' . $post_id . '&action=edit">' . $name . '</a>';
			}
			echo $title;
		} elseif($column == 'email') {
			echo get_post_meta( $post_id, 'email', true );
		}
	}

	function playersComments() {
		global $post;

		$data = array();

		$data['comments'] = get_post_meta( $post->ID, 'comments', true );

		echo $this->show('meta_box_players_comments', $data);
	}

	function playersMeta() {
		add_meta_box(
			$this->key_ . 'players_details',
			'Player Details',
			array($this, 'playersDetails'),
			$this->key_ . 'players',
			'normal',
			'default'
		);
		remove_meta_box( 'submitdiv', $this->key_ . 'players', 'side' );
		add_meta_box(
			$this->key_ . 'players' . '_save',
			'Save',
			array($this, 'metaSave'),
			$this->key_ . 'players',
			'side',
			'high'
		);
		add_meta_box(
			$this->key_ . 'players_comments',
			'Comments',
			array($this, 'playersComments'),
			$this->key_ . 'players',
			'side',
			'default'
		);
	}

	function playersDetails() {
		global $post;

		$data = array();

		$data['first_name'] = get_post_meta( $post->ID, 'first_name', true );
		$data['last_name'] = get_post_meta( $post->ID, 'last_name', true );
		$data['email'] = get_post_meta( $post->ID, 'email', true );
		$data['phone'] = get_post_meta( $post->ID, 'phone', true );
		$data['address'] = get_post_meta( $post->ID, 'address', true );
		$data['city'] = get_post_meta( $post->ID, 'city', true );
		$data['state'] = get_post_meta( $post->ID, 'state', true );
		$data['zip'] = get_post_meta( $post->ID, 'zip', true );
		$data['birth_year'] = get_post_meta( $post->ID, 'birth_year', true );
		$data['pdga'] = get_post_meta( $post->ID, 'pdga', true );

		$json_fn_ln_email = array();
		$json_pdga = array();
		$posts = get_posts([
			'post_type' => $this->key_ . 'players',
			'post_status' => 'publish',
			'numberposts' => -1,
			'exclude' =>  $post->ID,
			'fields' => 'ids'
		]);
		for($i = 0; $i < count($posts); $i++) {
			$fn_ln_email = get_post_meta($posts[$i], 'fn_ln_email', true);
			$pdga = get_post_meta($posts[$i], 'pdga', true);
			array_push($json_fn_ln_email, $fn_ln_email);
			array_push($json_pdga, $pdga);
		}
		$data['json_fn_ln_email'] =  json_encode($json_fn_ln_email);
		$data['json_pdga'] =  json_encode($json_pdga);

		echo $this->show('meta_box_players_details', $data);
	}

	function playersSave($post_id) {
		global $wpdb;
		if(!empty($_POST)) {
			update_post_meta($post_id, 'first_name', sanitize_text_field( $_POST['first_name']));
			update_post_meta($post_id, 'last_name', sanitize_text_field( $_POST['last_name']));
			update_post_meta($post_id, 'email', sanitize_text_field( $_POST['email']));
			update_post_meta($post_id, 'phone', sanitize_text_field( $_POST['phone']));
			update_post_meta($post_id, 'address', sanitize_text_field( $_POST['address']));
			update_post_meta($post_id, 'city', sanitize_text_field( $_POST['city']));
			update_post_meta($post_id, 'state', sanitize_text_field( $_POST['state']));
			update_post_meta($post_id, 'zip', sanitize_text_field( $_POST['zip']));
			update_post_meta($post_id, 'birth_year', sanitize_text_field( $_POST['birth_year']));
			update_post_meta($post_id, 'pdga', sanitize_text_field( $_POST['pdga']));

			update_post_meta($post_id, 'comments', sanitize_text_field( $_POST['comments']));

			$fn_ln_email = '';
			$fn_ln_email .= preg_replace('/[^a-z,0-9]/', '', strtolower(sanitize_text_field( $_POST['first_name'])));
			$fn_ln_email .= '_';
			$fn_ln_email .= preg_replace('/[^a-z,0-9]/', '', strtolower(sanitize_text_field( $_POST['last_name'])));
			$fn_ln_email .= '_';
			$fn_ln_email .= strtolower(sanitize_text_field( $_POST['email']));
			update_post_meta($post_id, 'fn_ln_email', $fn_ln_email);

			$first = sanitize_text_field($_POST['first_name']);
			$last = sanitize_text_field($_POST['last_name']);
			$name = $first . ' ' . $last;
			if($name == ' ') {
				$name = '(no name)';
			}
			$where = array( 'ID' => $post_id );
			$wpdb->update( $wpdb->posts, array( 'post_title' => $name ), $where );
		}
	}

	function preGetDocumentTitle($title) {
		global $post;
		if($this->first_time && has_shortcode($post->post_content, 'disc_golf_player') && isset($_GET['player']) && is_numeric($_GET['player'])) {
			$title = get_the_title($_GET['player']);
			$this->first_time = true;
			return $title;
		} elseif($this->first_time && has_shortcode($post->post_content, 'disc_golf_event') && isset($_GET['event']) && is_numeric($_GET['event'])) {
			$title = get_the_title($_GET['event']);
			$this->first_time = true;
			return $title;
		} else {
			return $title;
		}
	}

	// https://wordpress.stackexchange.com/a/70220
	// https://wpdreamer.com/2014/04/how-to-make-your-wordpress-admin-columns-sortable/
	function postsClauses($pieces, $query) {
		global $typenow, $wpdb;
		//echo $typenow;die;

		$scores = array('event', 'division');
		$events = array('start_date');
		if($typenow == $this->key_ . 'scores' && isset($query->query['orderby']) && in_array($query->query['orderby'], $scores)) {
			// Get the order query variable - ASC or DESC
			$order = strtoupper( $query->get( 'order' ) );

			// Make sure the order setting qualifies. If not, set default as ASC
			if ( ! in_array( $order, array( 'ASC', 'DESC' ) ) ) {
				$order = 'ASC';
			}

			$pieces[ 'join' ] .= " LEFT JOIN $wpdb->postmeta {$this->key}_pm ON {$this->key}_pm.post_id = {$wpdb->posts}.ID AND {$this->key}_pm.meta_key = '{$query->query['orderby']}'";
			$pieces[ 'join' ] .= " LEFT JOIN $wpdb->posts {$this->key}_p ON  {$this->key}_pm.meta_value = {$this->key}_p.ID";

			$pieces[ 'orderby' ] = "{$this->key}_p.post_title $order, " . $pieces[ 'orderby' ];
		} elseif($typenow == $this->key_ . 'events' && isset($query->query['orderby']) && in_array($query->query['orderby'], $events)) {
			// Get the order query variable - ASC or DESC
			$order = strtoupper( $query->get( 'order' ) );

			// Make sure the order setting qualifies. If not, set default as ASC
			if ( ! in_array( $order, array( 'ASC', 'DESC' ) ) ) {
				$order = 'ASC';
			}

			$pieces[ 'join' ] .= " LEFT JOIN $wpdb->postmeta {$this->key}_pm ON {$this->key}_pm.post_id = {$wpdb->posts}.ID AND {$this->key}_pm.meta_key = '{$query->query['orderby']}'";
			$pieces[ 'join' ] .= " LEFT JOIN $wpdb->posts {$this->key}_p ON  {$this->key}_pm.meta_value = {$this->key}_p.ID";

			$pieces[ 'orderby' ] = "{$this->key}_pm.meta_value $order, " . $pieces[ 'orderby' ];
		}

		return $pieces;
	}

	function postRowActions($actions, $post) {
		if(
			$post->post_type == $this->key_ . 'events'
			|| $post->post_type == $this->key_ . 'leagues'
			|| $post->post_type == $this->key_ . 'players'
			|| $post->post_type == $this->key_ . 'scores'
		) {
			unset($actions['edit']);
			unset($actions['trash']);
			$actions['inline hide-if-no-js'] = '';
		} elseif(
			$post->post_type == $this->key_ . 'divisions'
			|| $post->post_type == $this->key_ . 'seasons'
		) {
			unset($actions['edit']);
			unset($actions['trash']);
			unset($actions['inline hide-if-no-js']);
		}
		return $actions;
	}

	// From: https://wpdreamer.com/2014/04/how-to-make-your-wordpress-admin-columns-sortable/
	/* Now uses postsClauses()
	function preGetPosts($query) {
		if ( $query->is_main_query() && ( $orderby = $query->get( 'orderby' ) ) ) {
			if($orderby == 'event') {
				//echo '<pre>';
				//print_r($query);die;
				$query->set( 'meta_key', 'event' );
				$query->set( 'orderby', 'meta_value' );
			}
		}
	}
	 */

	// Inspired by: https://gist.github.com/davejamesmiller/1966595
	function redirectSave($location) {
		global $post;
		if($post->post_type == $this->key_ . 'players') {
			if(isset($_POST['initial_save']) && $_POST['initial_save'] == 'yes') {
				if(isset($_POST['redirect_redirect']) && $_POST['redirect_redirect'] == 'score') {
					$url = 'post-new.php?post_type=' . $this->key_ . 'scores';
					if(isset($_POST['redirect_event']) && is_numeric($_POST['redirect_event'])) {
						$url .= '&event=' . $_POST['redirect_event'];
					}
					if(isset($_POST['redirect_division']) && is_numeric($_POST['redirect_division'])) {
						$url .= '&division=' . $_POST['redirect_division'];
					}
					if(isset($_POST['redirect_player2']) && is_numeric($_POST['redirect_player2'])) {
						$url .= '&player2=' . $_POST['redirect_player2'];
					}
					$url .= '&player=' . $post->ID;
				} elseif(isset($_POST['redirect_redirect']) && $_POST['redirect_redirect'] == 'score2') {
					$url = 'post-new.php?post_type=' . $this->key_ . 'scores';
					if(isset($_POST['redirect_event']) && is_numeric($_POST['redirect_event'])) {
						$url .= '&event=' . $_POST['redirect_event'];
					}
					if(isset($_POST['redirect_division']) && is_numeric($_POST['redirect_division'])) {
						$url .= '&division=' . $_POST['redirect_division'];
					}
					if(isset($_POST['redirect_player']) && is_numeric($_POST['redirect_player'])) {
						$url .= '&player=' . $_POST['redirect_player'];
					}
					$url .= '&player2=' . $post->ID;
				} elseif(isset($_POST['redirect_redirect']) && $_POST['redirect_redirect'] == 'score-edit' && isset($_POST['redirect_post_id']) && is_numeric($_POST['redirect_post_id'])) {
					$url = 'post.php?post=' . $_POST['redirect_post_id'] . '&action=edit';
					if(isset($_POST['redirect_event']) && is_numeric($_POST['redirect_event'])) {
						$url .= '&event=' . $_POST['redirect_event'];
					}
					if(isset($_POST['redirect_division']) && is_numeric($_POST['redirect_division'])) {
						$url .= '&division=' . $_POST['redirect_division'];
					}
					if(isset($_POST['redirect_player2']) && is_numeric($_POST['redirect_player2'])) {
						$url .= '&player2=' . $_POST['redirect_player2'];
					}
					$url .= '&player=' . $post->ID;
				} elseif(isset($_POST['redirect_redirect']) && $_POST['redirect_redirect'] == 'score2-edit' && isset($_POST['redirect_post_id']) && is_numeric($_POST['redirect_post_id'])) {
					$url = 'post.php?post=' . $_POST['redirect_post_id'] . '&action=edit';
					if(isset($_POST['redirect_event']) && is_numeric($_POST['redirect_event'])) {
						$url .= '&event=' . $_POST['redirect_event'];
					}
					if(isset($_POST['redirect_division']) && is_numeric($_POST['redirect_division'])) {
						$url .= '&division=' . $_POST['redirect_division'];
					}
					if(isset($_POST['redirect_player']) && is_numeric($_POST['redirect_player'])) {
						$url .= '&player=' . $_POST['redirect_player'];
					}
					$url .= '&player2=' . $post->ID;
				} else {
					$url = 'post-new.php?post_type=' . $this->key_ . 'players';
				}
				$location = get_admin_url(null, $url);
				return $location;
			} else {
				$url = 'edit.php?post_type=' . $this->key_ . 'players';
				$location = get_admin_url(null, $url);
				return $location;
			}
		} elseif($post->post_type == $this->key_ . 'scores') {
			if(isset($_POST['initial_save']) && $_POST['initial_save'] == 'yes') {
				$url = 'post-new.php?post_type=' . $this->key_ . 'scores';
				if(isset($_POST['event']) && is_numeric($_POST['event'])) {
					$url .= '&event=' . $_POST['event'];
				}
				if(isset($_POST['division']) && is_numeric($_POST['division'])) {
					$url .= '&division=' . $_POST['division'];
				}
				$location = get_admin_url(null, $url);
				return $location;
			} else {
				$url = 'edit.php?post_type=' . $this->key_ . 'scores';
				$location = get_admin_url(null, $url);
				return $location;
			}
		} else {
			return $location;
		}
	}

	function scoresColumns() {
		$columns = array(
			'cb' => '<input type="checkbox" />',
			'title' => __( 'Name' ),
			'event' => __( 'Event' ),
			'division' => __( 'Division' ),
			'r1' => __( 'R1' ),
			'r2' => __( 'R2' ),
			'r3' => __( 'R3' ),
			'r4' => __( 'R4' ),
			'r5' => __( 'R5' ),
			'total' => __( 'Total' ),
			'winnings' => __( 'Winnings' ),
			'points' => __( 'Points' ),
			'rank' => __( 'Rank' ),
			'playoff_winner' => __( 'Play-off Winner' ),
		);

		return $columns;
	}

	function scoresColumnContent($column, $post_id) {
		if($column == 'event') {
			$event_id = get_post_meta( $post_id, 'event', true );
			$title = get_the_title($event_id);
			echo $title;
		} elseif($column == 'division') {
			$division_id = get_post_meta( $post_id, 'division', true );
			if($division_id) {
				$title = get_the_title($division_id);
				echo $title;
			} else {
				echo '';
			}
		} elseif($column == 'r1') {
			echo get_post_meta( $post_id, 'r1', true );
		} elseif($column == 'r2') {
			echo get_post_meta( $post_id, 'r2', true );
		} elseif($column == 'r3') {
			echo get_post_meta( $post_id, 'r3', true );
		} elseif($column == 'r4') {
			echo get_post_meta( $post_id, 'r4', true );
		} elseif($column == 'r5') {
			echo get_post_meta( $post_id, 'r5', true );
		} elseif($column == 'total') {
			echo get_post_meta( $post_id, 'total', true );
		} elseif($column == 'winnings') {
			echo get_post_meta( $post_id, 'winnings', true );
		} elseif($column == 'points') {
			echo get_post_meta( $post_id, 'points', true );
		} elseif($column == 'rank') {
			echo get_post_meta( $post_id, 'rank', true );
		} elseif($column == 'playoff_winner') {
			echo get_post_meta( $post_id, 'playoff_winner', true );
		}
	}

	function scoresColumnSort($cols) {
		return array(
			'title' => 'title',
			'event' => 'event',
			'division' => 'division',
		);
	}

	function scoresMeta() {
		add_meta_box(
			$this->key_ . 'scores_details',
			'Scores',
			array($this, 'scoresDetails'),
			$this->key_ . 'scores',
			'normal',
			'default'
		);
		remove_meta_box( 'submitdiv', $this->key_ . 'scores', 'side' );
		add_meta_box(
			$this->key_ . 'scores' . '_save',
			'Save',
			array($this, 'metaSave'),
			$this->key_ . 'scores',
			'side',
			'high'
		);
	}

	function scoresDetails() {
		global $post;
		$current_user = wp_get_current_user();

		$data = array();

		// From: https://gist.github.com/NiloySarker/2d1954eef3b0003d718d#file-replace-wp_submit-php-L93
		if (!in_array( $post->post_status, array('publish', 'future', 'private') ) || 0 == $post->ID ) {
			$data['initial_save'] = true;
		} else {
			$data['initial_save'] = false;
		}

		if(isset($_GET['event']) && is_numeric($_GET['event'])) {
			$selected = $_GET['event'];
		} else {
			$selected = get_post_meta( $post->ID, 'event', true );
		}

		if(in_array($this->key_ . 'tournament_director', $current_user->roles)) {
			$posts = get_posts([
				'author' => $current_user->ID,
				'post_type' => $this->key_ . 'events',
				'post_status' => 'publish',
				'numberposts' => -1,
				'fields' => 'ids'
			]);
			$data['events'] = wp_dropdown_pages(array(
				'show_option_none' => __( 'Please Select...' ),
				'post_type'=> $this->key_ . 'events',
				'name' => 'event',
				'echo' => 0,
				'include' => implode(',', $posts),
				'selected' => $selected
			));
		} else {
			$data['events'] = wp_dropdown_pages(array(
				'show_option_none' => __( 'Please Select...' ),
				'post_type'=> $this->key_ . 'events',
				'name' => 'event',
				'echo' => 0,
				'selected' => $selected
			));
		}

		if(isset($_GET['division']) && is_numeric($_GET['division'])) {
			$selected = $_GET['division'];
		} else {
			$selected = get_post_meta( $post->ID, 'division', true );
		}
		$data['divisions'] = wp_dropdown_pages(array(
			'show_option_none' => __( 'Please Select...' ),
			'post_type'=> $this->key_ . 'divisions',
			'name' => 'division',
			'echo' => 0,
			'selected' => $selected
		));

		if(isset($_GET['player']) && is_numeric($_GET['player'])) {
			$selected = $_GET['player'];
		} else {
			$selected = get_post_meta( $post->ID, 'player', true );
		}
		$data['players'] = wp_dropdown_pages(array(
			'show_option_none' => __( 'Please Select...' ),
			'post_type'=> $this->key_ . 'players',
			'name' => 'player',
			'echo' => 0,
			'selected' => $selected
		));

		if(isset($_GET['player2']) && is_numeric($_GET['player2'])) {
			$selected = $_GET['player2'];
		} else {
			$selected = get_post_meta( $post->ID, 'player2', true );
		}
		$data['players2'] = wp_dropdown_pages(array(
			'show_option_none' => __( 'Please Select...' ),
			'post_type'=> $this->key_ . 'players',
			'name' => 'player2',
			'echo' => 0,
			'selected' => $selected
		));

		$data['r1'] = get_post_meta( $post->ID, 'r1', true );
		$data['r2'] = get_post_meta( $post->ID, 'r2', true );
		$data['r3'] = get_post_meta( $post->ID, 'r3', true );
		$data['r4'] = get_post_meta( $post->ID, 'r4', true );
		$data['r5'] = get_post_meta( $post->ID, 'r5', true );

		$data['winnings'] = get_post_meta( $post->ID, 'winnings', true );

		$data['playoff_winner'] = get_post_meta( $post->ID, 'playoff_winner', true );


		$posts = get_posts([
			'post_type' => $this->key_ . 'scores',
			'post_status' => 'publish',
			'numberposts' => -1,
			'exclude' =>  $post->ID,
			'fields' => 'ids'
		]);
		$json = array();
		for($i = 0; $i < count($posts); $i++) {
			$event_id = get_post_meta($posts[$i], 'event', true);
			$player_id = get_post_meta($posts[$i], 'player', true);
			$player_id2 = get_post_meta($posts[$i], 'player2', true);

			if(!isset($json[$event_id])) {
				$json[$event_id] = array();
			}

			array_push($json[$event_id], $player_id);
			if($player_id2) {
				array_push($json[$event_id], $player_id2);
			}
		}

		$posts = get_posts([
			'post_type' => $this->key_ . 'events',
			'post_status' => 'publish',
			'numberposts' => -1,
			'exclude' =>  $post->ID,
			'fields' => 'ids',
			'meta_key' => 'doubles',
			'meta_value' => 'on',
		]);

		$data['json'] =  json_encode($json);
		$data['json_doubles_events'] =  json_encode($posts);

		$data['post_id'] =  $post->ID;

		echo $this->show('meta_box_scores_details', $data);
	}

	function scoresSave($post_id) {
		global $wpdb;
		if(!empty($_POST)) {
			$total = 0;
			$total += sanitize_text_field( $_POST['r1']) - 0;
			$total += sanitize_text_field( $_POST['r2']) - 0;
			$total += sanitize_text_field( $_POST['r3']) - 0;
			$total += sanitize_text_field( $_POST['r4']) - 0;
			$total += sanitize_text_field( $_POST['r5']) - 0;
			update_post_meta($post_id, 'event', sanitize_text_field( $_POST['event']));
			update_post_meta($post_id, 'division', sanitize_text_field( $_POST['division']));
			update_post_meta($post_id, 'player', sanitize_text_field( $_POST['player']));
			if(isset($_POST['player2'])) {
				update_post_meta($post_id, 'player2', sanitize_text_field( $_POST['player2']));
			}
			update_post_meta($post_id, 'r1', sanitize_text_field( $_POST['r1']));
			update_post_meta($post_id, 'r2', sanitize_text_field( $_POST['r2']));
			update_post_meta($post_id, 'r3', sanitize_text_field( $_POST['r3']));
			update_post_meta($post_id, 'r4', sanitize_text_field( $_POST['r4']));
			update_post_meta($post_id, 'r5', sanitize_text_field( $_POST['r5']));
			update_post_meta($post_id, 'total', $total);
			update_post_meta($post_id, 'winnings', sanitize_text_field( $_POST['winnings']));
			update_post_meta($post_id, 'playoff_winner', sanitize_text_field( $_POST['playoff_winner']));

			$this->updateScorePointsRank(sanitize_text_field( $_POST['event']), sanitize_text_field( $_POST['division']));

			// Set post_author to match event post_author
			// Deactivate save_post
			remove_action('save_post_' . $this->key_ . 'scores', array($this, 'scoresSave'));
			if(isset($_POST['event']) && is_numeric($_POST['event'])) {
				$post = get_post($_POST['event']);

				$args = array('ID' => $post_id, 'post_author' =>  $post->post_author);
				wp_update_post( $args );
			}
			// Reactivate save_post
			add_action('save_post_' . $this->key_ . 'scores', array($this, 'scoresSave'));

			if(isset($_POST['player2'])) {
				// Update title
				$player_id = sanitize_text_field($_POST['player']);
				$first = get_post_meta( $player_id, 'first_name', true );
				$last = get_post_meta( $player_id, 'last_name', true );
				$name = $first . ' ' . $last;
				$player_id2 = sanitize_text_field($_POST['player2']);
				if($player_id2) {
					$first = get_post_meta( $player_id2, 'first_name', true );
					$last = get_post_meta( $player_id2, 'last_name', true );
					$name .= ' | ' . $first . ' ' . $last;
				}
				if($name == ' ') {
					$name = '(no name)';
				}
				$where = array( 'ID' => $post_id );
				$wpdb->update( $wpdb->posts, array( 'post_title' => $name ), $where );
			} else {
				// Update title
				$player_id = sanitize_text_field($_POST['player']);
				$first = get_post_meta( $player_id, 'first_name', true );
				$last = get_post_meta( $player_id, 'last_name', true );
				$name = $first . ' ' . $last;
				if($name == ' ') {
					$name = '(no name)';
				}
				$where = array( 'ID' => $post_id );
				$wpdb->update( $wpdb->posts, array( 'post_title' => $name ), $where );
			}
		}
	}

	function seasonsColumns() {
		$columns = array(
			'cb' => '<input type="checkbox" />',
			'title' => __( 'Name' ),
		);

		return $columns;
	}

	function seasonsColumnContent($column, $post_id) {
		if($column == 'name') {
			$name = get_the_title($post_id);
			$title = '<a class="row-title" href="post.php?post=' . $post_id . '&action=edit">' . $name . '</a>';
			echo $title;
		}
	}

	function seasonsDetails() {
		global $post;

		$data = array();

		$data['start_date'] = get_post_meta( $post->ID, 'start_date', true );
		$data['end_date'] = get_post_meta( $post->ID, 'end_date', true );

		echo $this->show('meta_box_seasons_details', $data);
	}

	function seasonsMeta() {
		add_meta_box(
			$this->key_ . 'seasons_details',
			'Dates',
			array($this, 'seasonsDetails'),
			$this->key_ . 'seasons',
			'normal',
			'default'
		);
		add_meta_box(
			$this->key_ . 'seasons_points',
			'Points',
			array($this, 'seasonsPoints'),
			$this->key_ . 'seasons',
			'normal',
			'default'
		);
		remove_meta_box( 'submitdiv', $this->key_ . 'seasons', 'side' );
		add_meta_box(
			$this->key_ . 'seasons' . '_save',
			'Save',
			array($this, 'metaSave'),
			$this->key_ . 'seasons',
			'side',
			'high'
		);
	}

	function seasonsPoints() {
		global $post;

		$data = array();

		$data['points_participation'] = get_post_meta( $post->ID, 'points_participation', true );
		$data['points_per_player_beat'] = get_post_meta( $post->ID, 'points_per_player_beat', true );
		$data['points_division_win'] = get_post_meta( $post->ID, 'points_division_win', true );

		if(!$data['points_participation']) {
			$data['points_participation'] = 2;
		}
		if(!$data['points_per_player_beat']) {
			$data['points_per_player_beat'] = 1;
		}
		if(!$data['points_division_win']) {
			$data['points_division_win'] = 2;
		}

		echo $this->show('meta_box_seasons_points', $data);
	}

	function seasonsSave($post_id) {
		if(!empty($_POST)) {
			update_post_meta($post_id, 'start_date', sanitize_text_field( $_POST['start_date']));
			update_post_meta($post_id, 'end_date', sanitize_text_field( $_POST['end_date']));
			update_post_meta($post_id, 'points_participation', sanitize_text_field( $_POST['points_participation']));
			update_post_meta($post_id, 'points_per_player_beat', sanitize_text_field( $_POST['points_per_player_beat']));
			update_post_meta($post_id, 'points_division_win', sanitize_text_field( $_POST['points_division_win']));
			$this->updateAllPoints();
		}
	}

	function settings() {
		$data = array();

		$data['player'] = wp_dropdown_pages(array(
			'show_option_none' => __( 'Please Select...' ),
			'post_type'=> 'page',
			'name' => 'player',
			'echo' => 0,
			'selected' => get_option($this->key_ . 'player_page'),
		));

		$data['league'] = wp_dropdown_pages(array(
			'show_option_none' => __( 'Please Select...' ),
			'post_type'=> 'page',
			'name' => 'league',
			'echo' => 0,
			'selected' => get_option($this->key_ . 'league_page'),
		));

		$data['event'] = wp_dropdown_pages(array(
			'show_option_none' => __( 'Please Select...' ),
			'post_type'=> 'page',
			'name' => 'event',
			'echo' => 0,
			'selected' => get_option($this->key_ . 'event_page'),
		));

		echo $this->show('admin_settings', $data);
	}

	function shortAll() {
		// Get season and points
		$temp = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);
		$seasons = array();
		for($i = 0; $i < count($temp); $i++) {
			$season_start = get_post_meta( $temp[$i], 'start_date', true );
			$season_end = get_post_meta( $temp[$i], 'end_date', true );

			array_push($seasons, array('id' => $temp[$i], 'start' => $season_start, 'end' => $season_end ));
		}

		$score_ids = get_posts([
			'post_type' => $this->key_ . 'scores',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids'
		]);


		$scores = array();
		for($i = 0; $i < count($score_ids); $i++) {
			$post_id = $score_ids[$i];
			$event_id = get_post_meta( $post_id, 'event', true );
			$season_id = $this->shortGetSeason($seasons, $event_id);
			$division_id = get_post_meta( $post_id, 'division', true );
			$player_id = get_post_meta( $post_id, 'player', true );
			$player_id2 = get_post_meta( $post_id, 'player2', true );

			$full_id = $season_id . '_' . $division_id . '_' . $player_id;
			if(!isset($scores[$full_id])) {
				$season_name = get_the_title( $season_id);
				$division_name = get_the_title( $division_id);
				$name = get_the_title( $player_id);
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$temp = array();
				$temp['season_id'] = $season_id;
				$temp['division_id'] = $division_id;
				$temp['player_id'] = $player_id;
				$temp['season_name'] = $season_name;
				$temp['division_name'] = $division_name;
				$temp['name'] = $name;
				$temp['pdga'] = $pdga;
				$temp['events'] = 1;
				$temp['points'] = $points;
				$temp['winnings'] = $winnings;
				if($rank == 1) {
					$temp['first'] = 1;
					$temp['second'] = 0;
					$temp['third'] = 0;
				} elseif($rank == 2) {
					$temp['first'] = 0;
					$temp['second'] = 1;
					$temp['third'] = 0;
				} elseif($rank == 3) {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 1;
				} else {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 0;
				}
				$scores[$full_id] = $temp;
			} else {
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$scores[$full_id]['events'] += 1;
				$scores[$full_id]['points'] += $points;
				$scores[$full_id]['winnings'] += $winnings;
				if($rank == 1) {
					$scores[$full_id]['first'] += 1;
				} elseif($rank == 2) {
					$scores[$full_id]['second'] += 1;
				} elseif($rank == 3) {
					$scores[$full_id]['third'] += 1;
				}
			}


			if($player_id2) {
				$full_id = $season_id . '_' . $division_id . '_' . $player_id2;
				if(!isset($scores[$full_id])) {
					$season_name = get_the_title( $season_id);
					$division_name = get_the_title( $division_id);
					$name = get_the_title( $player_id2);
					$rank = get_post_meta( $post_id, 'rank', true );
					$points = get_post_meta( $post_id, 'points', true );
					$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
					$pdga = get_post_meta( $player_id2, 'pdga', true );

					$temp = array();
					$temp['season_id'] = $season_id;
					$temp['division_id'] = $division_id;
					$temp['player_id'] = $player_id2;
					$temp['season_name'] = $season_name;
					$temp['division_name'] = $division_name;
					$temp['name'] = $name;
					$temp['pdga'] = $pdga;
					$temp['events'] = 1;
					$temp['points'] = $points;
					$temp['winnings'] = $winnings;
					if($rank == 1) {
						$temp['first'] = 1;
						$temp['second'] = 0;
						$temp['third'] = 0;
					} elseif($rank == 2) {
						$temp['first'] = 0;
						$temp['second'] = 1;
						$temp['third'] = 0;
					} elseif($rank == 3) {
						$temp['first'] = 0;
						$temp['second'] = 0;
						$temp['third'] = 1;
					} else {
						$temp['first'] = 0;
						$temp['second'] = 0;
						$temp['third'] = 0;
					}
					$scores[$full_id] = $temp;
				} else {
					$rank = get_post_meta( $post_id, 'rank', true );
					$points = get_post_meta( $post_id, 'points', true );
					$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
					$pdga = get_post_meta( $player_id2, 'pdga', true );

					$scores[$full_id]['events'] += 1;
					$scores[$full_id]['points'] += $points;
					$scores[$full_id]['winnings'] += $winnings;
					if($rank == 1) {
						$scores[$full_id]['first'] += 1;
					} elseif($rank == 2) {
						$scores[$full_id]['second'] += 1;
					} elseif($rank == 3) {
						$scores[$full_id]['third'] += 1;
					}
				}
			}

		}

		return $scores;
	}

	function shortDivision($division_id) {
		// Get season and points
		$temp = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);
		$seasons = array();
		for($i = 0; $i < count($temp); $i++) {
			$season_start = get_post_meta( $temp[$i], 'start_date', true );
			$season_end = get_post_meta( $temp[$i], 'end_date', true );

			array_push($seasons, array('id' => $temp[$i], 'start' => $season_start, 'end' => $season_end ));
		}

		$score_ids = get_posts([
			'post_type' => $this->key_ . 'scores',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
			'meta_key' => 'division',
			'meta_value' => $division_id
		]);

		$score_ids2 = array();

		$scores = array();
		for($i = 0; $i < count($score_ids); $i++) {
			$post_id = $score_ids[$i];
			$event_id = get_post_meta( $post_id, 'event', true );
			$season_id = $this->shortGetSeason($seasons, $event_id);
			$division_id = get_post_meta( $post_id, 'division', true );
			$player_id = get_post_meta( $post_id, 'player', true );
			if(get_post_meta( $event_id, 'doubles', true )) {
				array_push($score_ids2, $score_ids[$i]);
			}

			$full_id = $season_id . '_' . $division_id . '_' . $player_id;

			if(!isset($scores[$full_id])) {
				$season_name = get_the_title( $season_id);
				$division_name = get_the_title( $division_id);
				$name = get_the_title( $player_id);
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$temp = array();
				$temp['season_id'] = $season_id;
				$temp['division_id'] = $division_id;
				$temp['player_id'] = $player_id;
				$temp['season_name'] = $season_name;
				$temp['division_name'] = $division_name;
				$temp['name'] = $name;
				$temp['pdga'] = $pdga;
				$temp['events'] = 1;
				$temp['points'] = $points;
				$temp['winnings'] = $winnings;
				if($rank == 1) {
					$temp['first'] = 1;
					$temp['second'] = 0;
					$temp['third'] = 0;
				} elseif($rank == 2) {
					$temp['first'] = 0;
					$temp['second'] = 1;
					$temp['third'] = 0;
				} elseif($rank == 3) {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 1;
				} else {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 0;
				}
				$scores[$full_id] = $temp;
			} else {
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$scores[$full_id]['events'] += 1;
				$scores[$full_id]['points'] += $points;
				$scores[$full_id]['winnings'] += $winnings;
				if($rank == 1) {
					$scores[$full_id]['first'] += 1;
				} elseif($rank == 2) {
					$scores[$full_id]['second'] += 1;
				} elseif($rank == 3) {
					$scores[$full_id]['third'] += 1;
				}
			}

		}
		for($i = 0; $i < count($score_ids2); $i++) {
			$post_id = $score_ids2[$i];
			$event_id = get_post_meta( $post_id, 'event', true );
			$season_id = $this->shortGetSeason($seasons, $event_id);
			$division_id = get_post_meta( $post_id, 'division', true );
			$player_id = get_post_meta( $post_id, 'player2', true );

			$full_id = $season_id . '_' . $division_id . '_' . $player_id;

			if(!isset($scores[$full_id])) {
				$season_name = get_the_title( $season_id);
				$division_name = get_the_title( $division_id);
				$name = get_the_title( $player_id);
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$temp = array();
				$temp['season_id'] = $season_id;
				$temp['division_id'] = $division_id;
				$temp['player_id'] = $player_id;
				$temp['season_name'] = $season_name;
				$temp['division_name'] = $division_name;
				$temp['name'] = $name;
				$temp['pdga'] = $pdga;
				$temp['events'] = 1;
				$temp['points'] = $points;
				$temp['winnings'] = $winnings;
				if($rank == 1) {
					$temp['first'] = 1;
					$temp['second'] = 0;
					$temp['third'] = 0;
				} elseif($rank == 2) {
					$temp['first'] = 0;
					$temp['second'] = 1;
					$temp['third'] = 0;
				} elseif($rank == 3) {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 1;
				} else {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 0;
				}
				$scores[$full_id] = $temp;
			} else {
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$scores[$full_id]['events'] += 1;
				$scores[$full_id]['points'] += $points;
				$scores[$full_id]['winnings'] += $winnings;
				if($rank == 1) {
					$scores[$full_id]['first'] += 1;
				} elseif($rank == 2) {
					$scores[$full_id]['second'] += 1;
				} elseif($rank == 3) {
					$scores[$full_id]['third'] += 1;
				}
			}

		}

		return $scores;
	}

	function shortEvent($event_id) {
		// Get season and points
		$temp = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);
		$seasons = array();
		for($i = 0; $i < count($temp); $i++) {
			$season_start = get_post_meta( $temp[$i], 'start_date', true );
			$season_end = get_post_meta( $temp[$i], 'end_date', true );

			array_push($seasons, array('id' => $temp[$i], 'start' => $season_start, 'end' => $season_end ));
		}

		$score_ids = get_posts([
			'post_type' => $this->key_ . 'scores',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
			'meta_key' => 'event',
			'meta_value' => $event_id
		]);


		$event_par = get_post_meta( $event_id, 'par', true ) - 0;

		$scores = array();
		for($i = 0; $i < count($score_ids); $i++) {
			$post_id = $score_ids[$i];
			$event_id = get_post_meta( $post_id, 'event', true );
			$season_id = $this->shortGetSeason($seasons, $event_id);
			$division_id = get_post_meta( $post_id, 'division', true );
			$player_id = get_post_meta( $post_id, 'player', true );
			if(get_post_meta( $event_id, 'doubles', true )) {
				$doubles = true;
				$player_id2 = get_post_meta( $post_id, 'player2', true );
				$name2 = get_the_title( $player_id2);
			} else {
				$doubles = false;
				$name2 = '';
			}

			$full_id = $season_id .'_' . $event_id . '_' . $division_id . '_' . $player_id;

			$season_name = get_the_title( $season_id);
			$event_name = get_the_title( $event_id);
			$division_name = get_the_title( $division_id);
			$name = get_the_title( $player_id);
			$rank = get_post_meta( $post_id, 'rank', true );
			$points = get_post_meta( $post_id, 'points', true );
			$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
			$pdga = get_post_meta( $player_id, 'pdga', true );
			$r1 = get_post_meta( $post_id, 'r1', true ) - 0;
			$r2 = get_post_meta( $post_id, 'r2', true ) - 0;
			$r3 = get_post_meta( $post_id, 'r3', true ) - 0;
			$r4 = get_post_meta( $post_id, 'r4', true ) - 0;
			$r5 = get_post_meta( $post_id, 'r5', true ) - 0;
			$total = get_post_meta( $post_id, 'total', true );
			if($event_par == 0) {
				$par = '';
			} else {
				$par = $total - $event_par;
			}

			if($doubles && $player_id2) {
				$pdga2 = get_post_meta( $player_id2, 'pdga', true );
			} else {
				$player_id2 = 0;
				$pdga2 = '';
			}

			$temp = array();
			$temp['season_id'] = $season_id;
			$temp['event_id'] = $event_id;
			$temp['division_id'] = $division_id;
			$temp['player_id'] = $player_id;
			$temp['player_id2'] = $player_id2;
			$temp['season_name'] = $season_name;
			$temp['event_name'] = $event_name;
			$temp['division_name'] = $division_name;
			$temp['name'] = $name;
			$temp['name2'] = $name2;
			$temp['pdga'] = $pdga;
			$temp['events'] = 1;
			$temp['points'] = $points;
			//$temp['winnings'] = '$' . number_format($winnings, 2);
			$temp['winnings'] = $winnings;
			$temp['rank'] = $rank;
			$temp['r1'] = $r1;
			$temp['r2'] = $r2;
			$temp['r3'] = $r3;
			$temp['r4'] = $r4;
			$temp['r5'] = $r5;
			$temp['total'] = $total;
			$temp['par'] = $par;
			$temp['doubles'] = $doubles;
			$temp['pdga2'] = $pdga2;
			$scores[$full_id] = $temp;
		}

		return $scores;
	}

	function shortEvents() {
		// Get season and points
		$temp = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);
		$seasons = array();
		for($i = 0; $i < count($temp); $i++) {
			$season_start = get_post_meta( $temp[$i], 'start_date', true );
			$season_end = get_post_meta( $temp[$i], 'end_date', true );

			array_push($seasons, array('id' => $temp[$i], 'start' => $season_start, 'end' => $season_end ));
		}

		$event_ids = get_posts([
			'post_type' => $this->key_ . 'events',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);


		$events = array();
		for($i = 0; $i < count($event_ids); $i++) {
			$event_id = $event_ids[$i];
			$season_id = $this->shortGetSeason($seasons, $event_id);
			$event_start = get_post_meta( $event_id, 'start_date', true );
			$score_ids = get_posts([
				'post_type' => $this->key_ . 'scores',
				'post_status' => 'publish',
				'numberposts' => -1,
				'fields' => 'ids',
				'meta_key' => 'event',
				'meta_value' => $event_id
			]);
			$players = count($score_ids);

			$full_id = $season_id .'_' . $event_start;

			$season_name = get_the_title( $season_id);
			$event_name = get_the_title( $event_id);
			$course_name = get_post_meta( $event_id, 'course_name', true );

			$temp = array();
			$temp['season_id'] = $season_id;
			$temp['event_id'] = $event_id;
			$temp['season_name'] = $season_name;
			$temp['event_name'] = $event_name;
			$temp['course_name'] = $course_name;
			$temp['start_date'] = date('F j, Y', strtotime($event_start));
			$temp['players'] = $players;
			$events[$full_id] = $temp;
		}

		return $events;
	}

	function shortFutureEvents() {
		// Get season and points
		$temp = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);
		$seasons = array();
		for($i = 0; $i < count($temp); $i++) {
			$season_start = get_post_meta( $temp[$i], 'start_date', true );
			$season_end = get_post_meta( $temp[$i], 'end_date', true );

			array_push($seasons, array('id' => $temp[$i], 'start' => $season_start, 'end' => $season_end ));
		}

		$event_ids = get_posts([
			'post_type' => $this->key_ . 'events',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);

		$now = date('Y-m-d H:i:s');

		$events = array();
		for($i = 0; $i < count($event_ids); $i++) {
			$event_id = $event_ids[$i];
			$event_start = get_post_meta( $event_id, 'start_date', true );
			if(strtotime($event_start) >= strtotime($now)) {
				$season_id = $this->shortGetSeason($seasons, $event_id);

				$full_id = $season_id .'_' . $event_start;

				$season_name = get_the_title( $season_id);
				$event_name = get_the_title( $event_id);
				$course_name = get_post_meta( $event_id, 'course_name', true );

				$temp = array();
				$temp['season_id'] = $season_id;
				$temp['event_id'] = $event_id;
				$temp['season_name'] = $season_name;
				$temp['event_name'] = $event_name;
				$temp['course_name'] = $course_name;
				$temp['start_date'] = date('F j, Y', strtotime($event_start));
				$events[$full_id] = $temp;
			}
		}

		return $events;
	}

	function shortGetActiveSeasons() {
		$season_ids = array();

		$now = date('Y-m-d H:i:s');

		$temp = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);
		$seasons = array();
		for($i = 0; $i < count($temp); $i++) {
			$season_start = get_post_meta( $temp[$i], 'start_date', true );
			$season_end = get_post_meta( $temp[$i], 'end_date', true );

			array_push($seasons, array('id' => $temp[$i], 'start' => $season_start, 'end' => $season_end ));
		}

		for($i = 0; $i < count($seasons); $i++) {
			if(strtotime($seasons[$i]['end']) >= strtotime($now)) {
				array_push($season_ids, $seasons[$i]['id']);
			}
		}

		return $season_ids;
	}

	function shortGetSeason($seasons, $event_id) {
		$season_id = 0;

		// Get event start
		$event_date = get_post_meta( $event_id, 'start_date', true );

		for($i = 0; $i < count($seasons); $i++) {

			if(strtotime($seasons[$i]['start']) <= strtotime($event_date) && strtotime($event_date) <= strtotime($seasons[$i]['end'])) {
				$season_id = $seasons[$i]['id'];
				break;
			}
		}

		return $season_id;
	}

	function shortLeague($league_id) {
		// Get season and points
		$temp = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);
		$seasons = array();
		for($i = 0; $i < count($temp); $i++) {
			$season_start = get_post_meta( $temp[$i], 'start_date', true );
			$season_end = get_post_meta( $temp[$i], 'end_date', true );

			array_push($seasons, array('id' => $temp[$i], 'start' => $season_start, 'end' => $season_end ));
		}

		$event_ids = get_posts([
			'post_type' => $this->key_ . 'events',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
			'meta_key' => 'league',
			'meta_value' => $league_id
		]);

		if(count($event_ids) == 0) {
			return array();
		}

		$score_ids = get_posts([
			'post_type' => $this->key_ . 'scores',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
			'meta_query' => array(
				array(
					'key' => 'event',
					'value' => $event_ids,
					'compare' => 'IN'
				)
			)
		]);

		$score_ids2 = array();

		$scores = array();
		for($i = 0; $i < count($score_ids); $i++) {
			$post_id = $score_ids[$i];
			$event_id = get_post_meta( $post_id, 'event', true );
			$season_id = $this->shortGetSeason($seasons, $event_id);
			$division_id = get_post_meta( $post_id, 'division', true );
			$player_id = get_post_meta( $post_id, 'player', true );
			if(get_post_meta( $event_id, 'doubles', true )) {
				array_push($score_ids2, $score_ids[$i]);
			}

			$full_id = $season_id . '_' . $division_id . '_' . $player_id;

			if(!isset($scores[$full_id])) {
				$season_name = get_the_title( $season_id);
				$division_name = get_the_title( $division_id);
				$name = get_the_title( $player_id);
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$temp = array();
				$temp['season_id'] = $season_id;
				$temp['division_id'] = $division_id;
				$temp['player_id'] = $player_id;
				$temp['season_name'] = $season_name;
				$temp['division_name'] = $division_name;
				$temp['name'] = $name;
				$temp['pdga'] = $pdga;
				$temp['events'] = 1;
				$temp['points'] = $points;
				$temp['winnings'] = $winnings;
				if($rank == 1) {
					$temp['first'] = 1;
					$temp['second'] = 0;
					$temp['third'] = 0;
				} elseif($rank == 2) {
					$temp['first'] = 0;
					$temp['second'] = 1;
					$temp['third'] = 0;
				} elseif($rank == 3) {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 1;
				} else {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 0;
				}
				$scores[$full_id] = $temp;
			} else {
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$scores[$full_id]['events'] += 1;
				$scores[$full_id]['points'] += $points;
				$scores[$full_id]['winnings'] += $winnings;
				if($rank == 1) {
					$scores[$full_id]['first'] += 1;
				} elseif($rank == 2) {
					$scores[$full_id]['second'] += 1;
				} elseif($rank == 3) {
					$scores[$full_id]['third'] += 1;
				}
			}

		}
		for($i = 0; $i < count($score_ids2); $i++) {
			$post_id = $score_ids2[$i];
			$event_id = get_post_meta( $post_id, 'event', true );
			$season_id = $this->shortGetSeason($seasons, $event_id);
			$division_id = get_post_meta( $post_id, 'division', true );
			$player_id = get_post_meta( $post_id, 'player2', true );

			$full_id = $season_id . '_' . $division_id . '_' . $player_id;

			if(!isset($scores[$full_id])) {
				$season_name = get_the_title( $season_id);
				$division_name = get_the_title( $division_id);
				$name = get_the_title( $player_id);
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$temp = array();
				$temp['season_id'] = $season_id;
				$temp['division_id'] = $division_id;
				$temp['player_id'] = $player_id;
				$temp['season_name'] = $season_name;
				$temp['division_name'] = $division_name;
				$temp['name'] = $name;
				$temp['pdga'] = $pdga;
				$temp['events'] = 1;
				$temp['points'] = $points;
				$temp['winnings'] = $winnings;
				if($rank == 1) {
					$temp['first'] = 1;
					$temp['second'] = 0;
					$temp['third'] = 0;
				} elseif($rank == 2) {
					$temp['first'] = 0;
					$temp['second'] = 1;
					$temp['third'] = 0;
				} elseif($rank == 3) {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 1;
				} else {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 0;
				}
				$scores[$full_id] = $temp;
			} else {
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$scores[$full_id]['events'] += 1;
				$scores[$full_id]['points'] += $points;
				$scores[$full_id]['winnings'] += $winnings;
				if($rank == 1) {
					$scores[$full_id]['first'] += 1;
				} elseif($rank == 2) {
					$scores[$full_id]['second'] += 1;
				} elseif($rank == 3) {
					$scores[$full_id]['third'] += 1;
				}
			}

		}

		return $scores;
	}

	function shortLeagueEvents($league_id) {
		// Get events
		$event_ids = get_posts([
			'post_type' => $this->key_ . 'events',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
			'meta_key' => 'league',
			'meta_value' => $league_id
		]);

		if(count($event_ids) == 0) {
			return array();
		}

		$events = array();
		for($i = 0; $i < count($event_ids); $i++) {
			$event_id = $event_ids[$i];

			$event_name = get_the_title( $event_id);

			$full_id = $league_id . '_' . $event_id;

			$temp = array();
			$temp['event_id'] = $event_id;
			$temp['event_name'] = $event_name;
			$events[$full_id] = $temp;
		}

		return $events;
	}

	function shortLeagues() {
		// Get season and points
		$temp = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);
		$seasons = array();
		for($i = 0; $i < count($temp); $i++) {
			$season_start = get_post_meta( $temp[$i], 'start_date', true );
			$season_end = get_post_meta( $temp[$i], 'end_date', true );

			array_push($seasons, array('id' => $temp[$i], 'start' => $season_start, 'end' => $season_end ));
		}

		$league_ids = get_posts([
			'post_type' => $this->key_ . 'leagues',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids'
		]);

		$leagues = array();
		for($i = 0; $i < count($league_ids); $i++) {
			$league_id = $league_ids[$i];
			$season_id = get_post_meta( $league_id, 'season', true );
			$season_name = get_the_title( $season_id);
			$event_ids = get_posts([
				'post_type' => $this->key_ . 'events',
				'post_status' => 'publish',
				'numberposts' => -1,
				'fields' => 'ids',
				'meta_key' => 'league',
				'meta_value' => $league_id
			]);
			$full_id = $season_id . '_' . $league_id;

			$temp = array();
			$temp['season_id'] = $season_id;
			$temp['league_id'] = $league_id;
			$temp['season_name'] = $season_name;
			$temp['league_name'] = get_the_title($league_id);
			$temp['total_events'] = count($event_ids);

			$leagues[$full_id] = $temp;
		}

		return $leagues;
	}

	function shortLink($post_id) {
		$output = get_page_link($post_id);
		if(strpos($output, '?') === false) {
			$output .= '?';
		} else {
			$output .= '&';
		}
		return $output;
	}

	function shortPlayerDivisions($player_id) {
		// Get season and points
		$temp = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);
		$seasons = array();
		for($i = 0; $i < count($temp); $i++) {
			$season_start = get_post_meta( $temp[$i], 'start_date', true );
			$season_end = get_post_meta( $temp[$i], 'end_date', true );

			array_push($seasons, array('id' => $temp[$i], 'start' => $season_start, 'end' => $season_end ));
		}

		$score_ids = get_posts([
			'post_type' => $this->key_ . 'scores',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
			'meta_key' => 'player',
			'meta_value' => $player_id
		]);

		$score_ids2 = get_posts([
			'post_type' => $this->key_ . 'scores',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
			'meta_key' => 'player2',
			'meta_value' => $player_id
		]);

		$score_ids = array_merge($score_ids, $score_ids2);

		$scores = array();
		for($i = 0; $i < count($score_ids); $i++) {
			$post_id = $score_ids[$i];
			$event_id = get_post_meta( $post_id, 'event', true );
			$season_id = $this->shortGetSeason($seasons, $event_id);
			$division_id = get_post_meta( $post_id, 'division', true );
			$player_id = get_post_meta( $post_id, 'player', true );

			$full_id = $season_id . '_' . $division_id . '_' . $player_id;

			if(!isset($scores[$full_id])) {
				$season_name = get_the_title( $season_id);
				$division_name = get_the_title( $division_id);
				$name = get_the_title( $player_id);
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$temp = array();
				$temp['season_id'] = $season_id;
				$temp['division_id'] = $division_id;
				$temp['player_id'] = $player_id;
				$temp['season_name'] = $season_name;
				$temp['division_name'] = $division_name;
				$temp['name'] = $name;
				$temp['pdga'] = $pdga;
				$temp['events'] = 1;
				$temp['points'] = $points;
				$temp['winnings'] = $winnings;
				if($rank == 1) {
					$temp['first'] = 1;
					$temp['second'] = 0;
					$temp['third'] = 0;
				} elseif($rank == 2) {
					$temp['first'] = 0;
					$temp['second'] = 1;
					$temp['third'] = 0;
				} elseif($rank == 3) {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 1;
				} else {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 0;
				}
				$scores[$full_id] = $temp;
			} else {
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$scores[$full_id]['events'] += 1;
				$scores[$full_id]['points'] += $points;
				$scores[$full_id]['winnings'] += $winnings;
				if($rank == 1) {
					$scores[$full_id]['first'] += 1;
				} elseif($rank == 2) {
					$scores[$full_id]['second'] += 1;
				} elseif($rank == 3) {
					$scores[$full_id]['third'] += 1;
				}
			}

		}

		return $scores;
	}

	function shortPlayerEvents($player_id) {
		// Get season and points
		$temp = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);
		$seasons = array();
		for($i = 0; $i < count($temp); $i++) {
			$season_start = get_post_meta( $temp[$i], 'start_date', true );
			$season_end = get_post_meta( $temp[$i], 'end_date', true );

			array_push($seasons, array('id' => $temp[$i], 'start' => $season_start, 'end' => $season_end ));
		}

		$score_ids = get_posts([
			'post_type' => $this->key_ . 'scores',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
			'meta_key' => 'player',
			'meta_value' => $player_id
		]);

		$score_ids2 = get_posts([
			'post_type' => $this->key_ . 'scores',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
			'meta_key' => 'player2',
			'meta_value' => $player_id
		]);

		$score_ids = array_merge($score_ids, $score_ids2);

		$scores = array();
		for($i = 0; $i < count($score_ids); $i++) {
			$post_id = $score_ids[$i];
			$event_id = get_post_meta( $post_id, 'event', true );
			$season_id = $this->shortGetSeason($seasons, $event_id);
			$division_id = get_post_meta( $post_id, 'division', true );
			$player_id = get_post_meta( $post_id, 'player', true );

			$full_id = $season_id .'_' . $event_id . '_' . $division_id . '_' . $player_id;

			$season_name = get_the_title( $season_id);
			$event_name = get_the_title( $event_id);
			$division_name = get_the_title( $division_id);
			$name = get_the_title( $player_id);
			$rank = get_post_meta( $post_id, 'rank', true );
			$points = get_post_meta( $post_id, 'points', true );
			$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
			$pdga = get_post_meta( $player_id, 'pdga', true );
			$r1 = get_post_meta( $post_id, 'r1', true ) - 0;
			$r2 = get_post_meta( $post_id, 'r2', true ) - 0;
			$r3 = get_post_meta( $post_id, 'r3', true ) - 0;
			$r4 = get_post_meta( $post_id, 'r4', true ) - 0;
			$r5 = get_post_meta( $post_id, 'r5', true ) - 0;
			$total = get_post_meta( $post_id, 'total', true );

			$temp = array();
			$temp['season_id'] = $season_id;
			$temp['event_id'] = $event_id;
			$temp['division_id'] = $division_id;
			$temp['player_id'] = $player_id;
			$temp['season_name'] = $season_name;
			$temp['event_name'] = $event_name;
			$temp['division_name'] = $division_name;
			$temp['name'] = $name;
			$temp['pdga'] = $pdga;
			$temp['events'] = 1;
			$temp['points'] = $points;
			//$temp['winnings'] = '$' . number_format($winnings, 2);
			$temp['winnings'] = $winnings;
			$temp['rank'] = $rank;
			$temp['r1'] = $r1;
			$temp['r2'] = $r2;
			$temp['r3'] = $r3;
			$temp['r4'] = $r4;
			$temp['r5'] = $r5;
			$temp['total'] = $total;
			$scores[$full_id] = $temp;
		}

		return $scores;
	}

	function shortPlayers() {
		// Get season and points
		$temp = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);
		$seasons = array();
		for($i = 0; $i < count($temp); $i++) {
			$season_start = get_post_meta( $temp[$i], 'start_date', true );
			$season_end = get_post_meta( $temp[$i], 'end_date', true );

			array_push($seasons, array('id' => $temp[$i], 'start' => $season_start, 'end' => $season_end ));
		}

		$score_ids = get_posts([
			'post_type' => $this->key_ . 'scores',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids'
		]);


		$scores = array();
		for($i = 0; $i < count($score_ids); $i++) {
			$post_id = $score_ids[$i];
			$event_id = get_post_meta( $post_id, 'event', true );
			$season_id = $this->shortGetSeason($seasons, $event_id);
			$division_id = get_post_meta( $post_id, 'division', true );
			$player_id = get_post_meta( $post_id, 'player', true );
			$player_id2 = get_post_meta( $post_id, 'player2', true );

			$full_id = $season_id . '_' . $player_id;
			if(!isset($scores[$full_id])) {
				$season_name = get_the_title( $season_id);
				$division_name = get_the_title( $division_id);
				$name = get_the_title( $player_id);
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$temp = array();
				$temp['season_id'] = $season_id;
				$temp['division_id'] = $division_id;
				$temp['player_id'] = $player_id;
				$temp['season_name'] = $season_name;
				$temp['division_name'] = $division_name;
				$temp['name'] = $name;
				$temp['pdga'] = $pdga;
				$temp['events'] = 1;
				$temp['points'] = $points;
				$temp['winnings'] = $winnings;
				if($rank == 1) {
					$temp['first'] = 1;
					$temp['second'] = 0;
					$temp['third'] = 0;
				} elseif($rank == 2) {
					$temp['first'] = 0;
					$temp['second'] = 1;
					$temp['third'] = 0;
				} elseif($rank == 3) {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 1;
				} else {
					$temp['first'] = 0;
					$temp['second'] = 0;
					$temp['third'] = 0;
				}
				$scores[$full_id] = $temp;
			} else {
				$rank = get_post_meta( $post_id, 'rank', true );
				$points = get_post_meta( $post_id, 'points', true );
				$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
				$pdga = get_post_meta( $player_id, 'pdga', true );

				$scores[$full_id]['events'] += 1;
				$scores[$full_id]['points'] += $points;
				$scores[$full_id]['winnings'] += $winnings;
				if($rank == 1) {
					$scores[$full_id]['first'] += 1;
				} elseif($rank == 2) {
					$scores[$full_id]['second'] += 1;
				} elseif($rank == 3) {
					$scores[$full_id]['third'] += 1;
				}
			}


			if($player_id2) {
				$full_id = $season_id . '_' . $player_id2;
				if(!isset($scores[$full_id])) {
					$season_name = get_the_title( $season_id);
					$division_name = get_the_title( $division_id);
					$name = get_the_title( $player_id2);
					$rank = get_post_meta( $post_id, 'rank', true );
					$points = get_post_meta( $post_id, 'points', true );
					$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
					$pdga = get_post_meta( $player_id2, 'pdga', true );

					$temp = array();
					$temp['season_id'] = $season_id;
					$temp['division_id'] = $division_id;
					$temp['player_id'] = $player_id2;
					$temp['season_name'] = $season_name;
					$temp['division_name'] = $division_name;
					$temp['name'] = $name;
					$temp['pdga'] = $pdga;
					$temp['events'] = 1;
					$temp['points'] = $points;
					$temp['winnings'] = $winnings;
					if($rank == 1) {
						$temp['first'] = 1;
						$temp['second'] = 0;
						$temp['third'] = 0;
					} elseif($rank == 2) {
						$temp['first'] = 0;
						$temp['second'] = 1;
						$temp['third'] = 0;
					} elseif($rank == 3) {
						$temp['first'] = 0;
						$temp['second'] = 0;
						$temp['third'] = 1;
					} else {
						$temp['first'] = 0;
						$temp['second'] = 0;
						$temp['third'] = 0;
					}
					$scores[$full_id] = $temp;
				} else {
					$rank = get_post_meta( $post_id, 'rank', true );
					$points = get_post_meta( $post_id, 'points', true );
					$winnings = get_post_meta( $post_id, 'winnings', true ) - 0;
					$pdga = get_post_meta( $player_id2, 'pdga', true );

					$scores[$full_id]['events'] += 1;
					$scores[$full_id]['points'] += $points;
					$scores[$full_id]['winnings'] += $winnings;
					if($rank == 1) {
						$scores[$full_id]['first'] += 1;
					} elseif($rank == 2) {
						$scores[$full_id]['second'] += 1;
					} elseif($rank == 3) {
						$scores[$full_id]['third'] += 1;
					}
				}
			}

		}

		return $scores;
	}


	function shortcodeDiscGolfDivision() {
		$data = array();
		if(isset($_GET['division']) && is_numeric($_GET['division'])) {
			$divisions = $this->shortDivision($_GET['division']);
		} else {
			$divisions = array();
		}
		$data['json'] = json_encode($divisions);
		$data['player_base_url'] = $this->shortLink(get_option($this->key_ . 'player_page'));

		echo $this->show('shortcode_disc_golf_division', $data);
	}

	function shortcodeDiscGolfEvent() {
		$data = array();
		if(isset($_GET['event']) && is_numeric($_GET['event'])) {
			$event_id = $_GET['event'];
			$event = $this->shortEvent($event_id);

			$start_date = get_post_meta( $event_id, 'start_date', true );
			$end_date = get_post_meta( $event_id, 'end_date', true );
			$date = '';
			$date .= date('F j, Y', strtotime($start_date));
			if($end_date) {
				$date .= ' to ';
				$date .= date('F j, Y', strtotime($end_date));
			}

			$course_name = get_post_meta( $event_id, 'course_name', true );
			$city = get_post_meta( $event_id, 'city', true );
			$state = get_post_meta( $event_id, 'state', true );
			$location = '';
			$location .= $course_name;
			if($city) {
				$location .= ' ' . $city . ', ';
			}
			$location .= ' ' . $state;

			$tournament_director_id = get_post_meta( $event_id, 'tournament_director', true );
			if($tournament_director_id) {
				$tournament_director = get_the_title( $tournament_director_id);
			} else {
				$tournament_director = '';
			}

			$data['date'] = $date;
			$data['location'] = $location;
			$data['tournament_director'] = $tournament_director;
			$data['total_players'] = count($event);
			$data['details'] = get_post_meta( $event_id, 'comments', true );
			$data['show_event'] = true;
			$events = false;
		} else {
			$data['show_event'] = false;
			$event = false;
			$events = $this->shortEvents();
		}
		$data['json_event'] = json_encode($event);
		$data['json_events'] = json_encode($events);
		$data['event_base_url'] = $this->shortLink(get_option($this->key_ . 'event_page'));
		$data['player_base_url'] = $this->shortLink(get_option($this->key_ . 'player_page'));

		echo $this->show('shortcode_disc_golf_event', $data);
	}

	function shortcodeDiscGolfFutureEvents() {
		$data = array();

		$events = $this->shortFutureEvents();
		$data['json_events'] = json_encode($events);

		$data['event_base_url'] = $this->shortLink(get_option($this->key_ . 'event_page'));

		echo $this->show('shortcode_disc_golf_future_events', $data);
	}

	function shortcodeDiscGolfLeague() {
		$data = array();
		if(isset($_GET['league']) && is_numeric($_GET['league'])) {
			$league_id = $_GET['league'];
			$league = $this->shortLeague($league_id);
			$leagues = false;

			$event_ids = get_posts([
				'post_type' => $this->key_ . 'events',
				'post_status' => 'publish',
				'numberposts' => -1,
				'fields' => 'ids',
				'meta_key' => 'league',
				'meta_value' => $league_id
			]);

			$data['show_details'] = true;
			$data['season'] = get_the_title($league_id);
			$data['total_events'] = count($event_ids);
			$events = $this->shortLeagueEvents($league_id);
		} else {
			$league = false;
			$leagues = $this->shortLeagues();
			$data['show_details'] = false;
			$events = false;
		}
		$data['json_league'] = json_encode($league);
		$data['json_leagues'] = json_encode($leagues);
		$data['json_events'] = json_encode($events);
		$data['event_base_url'] = $this->shortLink(get_option($this->key_ . 'event_page'));
		$data['league_base_url'] = $this->shortLink(get_option($this->key_ . 'league_page'));
		$data['player_base_url'] = $this->shortLink(get_option($this->key_ . 'player_page'));

		echo $this->show('shortcode_disc_golf_league', $data);
	}

	function shortcodeDiscGolfOverview() {
		$data = $this->shortAll();

		//echo '<pre>'; print_r($data); die;

		$data['json'] = json_encode($data);
		$data['player_base_url'] = $this->shortLink(get_option($this->key_ . 'player_page'));

		echo $this->show('shortcode_disc_golf_overview', $data);
	}

	function shortcodeDiscGolfPlayer() {
		$data = array();
		if(isset($_GET['player']) && is_numeric($_GET['player'])) {
			$divisions = $this->shortPlayerDivisions($_GET['player']);
			$events = $this->shortPlayerEvents($_GET['player']);
			$players = false;
		} else {
			$divisions = false;
			$events = false;
			$players = $this->shortPlayers();
		}
		$data['json_divisions'] = json_encode($divisions);
		$data['json_events'] = json_encode($events);
		$data['json_players'] = json_encode($players);

		$data['player_base_url'] = $this->shortLink(get_option($this->key_ . 'player_page'));
		$data['event_base_url'] = $this->shortLink(get_option($this->key_ . 'event_page'));

		echo $this->show('shortcode_disc_golf_player', $data);
	}

	function shortcodeDiscGolfPoints() {
		$data = $this->shortAll();

		//echo '<pre>'; print_r($data); die;

		$data['json'] = json_encode($data);
		$data['player_base_url'] = $this->shortLink(get_option($this->key_ . 'player_page'));

		echo $this->show('shortcode_disc_golf_points', $data);
	}

	// From: https://wordpress.stackexchange.com/a/147559
	function theTitle($title, $post_id = null) {
		global $post;
		if($post_id && get_post_type($post_id) == $this->key_ . 'scores') {
			$player_id = get_post_meta( $post_id, 'player', true );
			$first = get_post_meta( $player_id, 'first_name', true );
			$last = get_post_meta( $player_id, 'last_name', true );
			$name = $first . ' ' . $last;
			$player_id2 = get_post_meta( $post_id, 'player2', true );
			if($player_id2) {
				$first = get_post_meta( $player_id2, 'first_name', true );
				$last = get_post_meta( $player_id2, 'last_name', true );
				$name .= ' | ' . $first . ' ' . $last;
			}
			if($name == ' ') {
				$name = '(no name)';
			}
			return $name;
		} elseif($post_id && in_the_loop() && $this->first_time && has_shortcode($post->post_content, 'disc_golf_player') && isset($_GET['player']) && is_numeric($_GET['player'])) {
			$this->first_time = false;
			$title = get_the_title($_GET['player']);
			return $title;
		} elseif($post_id && in_the_loop() && $this->first_time && has_shortcode($post->post_content, 'disc_golf_event') && isset($_GET['event']) && is_numeric($_GET['event'])) {
			$this->first_time = false;
			$title = get_the_title($_GET['event']);
			return $title;
		} else {
			return $title;
		}
	}

	function updateAllPoints() {
		$events = get_posts([
			'post_type' => $this->key_ . 'events',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);
		$divisions = get_posts([
			'post_type' => $this->key_ . 'divisions',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);

		for($i = 0; $i < count($events); $i++) {
			for($j = 0; $j < count($divisions); $j++) {
				$this->updateScorePointsRank($events[$i], $divisions[$j]);
			}
		}
	}

	function updateEventPointsRank($event_id) {
		$event_date = get_post_meta( $event_id, 'start_date', true );

		$points_participation = 0;
		$points_per_player_beat = 0;
		$points_division_win = 0;

		$season_id = -1;

		// Get season and points
		$seasons = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);

		for($i = 0; $i < count($seasons); $i++) {
			$season_start = get_post_meta( $seasons[$i], 'start_date', true );
			$season_end = get_post_meta( $seasons[$i], 'end_date', true );

			if(strtotime($season_start) <= strtotime($event_date) && strtotime($event_date) <= strtotime($season_end)) {
				$season_id = $seasons[$i];
				break;
			}
		}

		$divisions = get_posts([
			'post_type' => $this->key_ . 'divisions',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);

		for($i = 0; $i < count($divisions); $i++) {
			$this->updateScorePointsRank($event_id, $divisions[$i]);
		}
	}

	function updateEventPoints($event_id, $last_event_date, $new_event_date) {
		$points_participation = 0;
		$points_per_player_beat = 0;
		$points_division_win = 0;

		$last_season_id = -1;
		$new_season_id = -1;

		// Get season and points
		$seasons = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);

		for($i = 0; $i < count($seasons); $i++) {
			$season_start = get_post_meta( $seasons[$i], 'start_date', true );
			$season_end = get_post_meta( $seasons[$i], 'end_date', true );

			if(strtotime($season_start) <= strtotime($last_event_date) && strtotime($last_event_date) <= strtotime($season_end)) {
				$last_season_id = $seasons[$i];
				break;
			}
		}
		for($i = 0; $i < count($seasons); $i++) {
			$season_start = get_post_meta( $seasons[$i], 'start_date', true );
			$season_end = get_post_meta( $seasons[$i], 'end_date', true );

			if(strtotime($season_start) <= strtotime($new_event_date) && strtotime($new_event_date) <= strtotime($season_end)) {
				$new_season_id = $seasons[$i];
				$points_participation = get_post_meta( $seasons[$i], 'points_participation', true );
				$points_per_player_beat = get_post_meta( $seasons[$i], 'points_per_player_beat', true );
				$points_division_win = get_post_meta( $seasons[$i], 'points_division_win', true );
				break;
			}
		}

		if($last_season_id != $new_season_id) {
			$divisions = get_posts([
				'post_type' => $this->key_ . 'divisions',
				'post_status' => 'publish',
				'numberposts' => -1,
				'fields' => 'ids',
			]);

			for($i = 0; $i < count($divisions); $i++) {
				$this->updateScorePointsRank($event_id, $divisions[$i]);
			}
		}
	}

	function updateScorePointsRank($event_id, $division_id) {
		$points_participation = 0;
		$points_per_player_beat = 0;
		$points_division_win = 0;

		// Get event start
		$event_date = get_post_meta( $event_id, 'start_date', true );

		// Get season and points
		$seasons = get_posts([
			'post_type' => $this->key_ . 'seasons',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
		]);

		for($i = 0; $i < count($seasons); $i++) {
			$season_start = get_post_meta( $seasons[$i], 'start_date', true );
			$season_end = get_post_meta( $seasons[$i], 'end_date', true );

			if(strtotime($season_start) <= strtotime($event_date) && strtotime($event_date) <= strtotime($season_end)) {
				$points_participation = get_post_meta( $seasons[$i], 'points_participation', true );
				$points_per_player_beat = get_post_meta( $seasons[$i], 'points_per_player_beat', true );
				$points_division_win = get_post_meta( $seasons[$i], 'points_division_win', true );
				break;
			}
		}

		$scores = array();
		$temp = get_posts([
			'post_type' => $this->key_ . 'scores',
			'post_status' => 'publish',
			'numberposts' => -1,
			'fields' => 'ids',
			'meta_query' => array(
				array(
					'key' => 'event',
					'value' => $event_id,
				),
				array(
					'key' => 'division',
					'value' => $division_id,
				)
			)
		]);

		for($i = 0; $i < count($temp); $i++) {
			$total = get_post_meta( $temp[$i], 'total', true );
			$playoff_winner = get_post_meta( $temp[$i], 'playoff_winner', true );

			array_push($scores, array('score_id' => $temp[$i], 'total' => $total, 'playoff_winner' => $playoff_winner));
		}

		usort($scores, function($a, $b) {
			if($a['playoff_winner'] == 'Yes' && $b['playoff_winner'] != 'Yes') {
				return -1;
			} elseif($b['playoff_winner'] == 'Yes' && $a['playoff_winner'] != 'Yes') {
				return 1;
			} else {
				return $a['total'] - $b['total'];
			}
		});

		$last_score = -1;
		$last_rank = 1;
		$winning_score = 0;
		for($i = 0; $i < count($scores); $i++) {
			if($i == 0) {
				$winning_score = $scores[$i]['total'];
			}

			$points = 0;
			$points += $points_participation;

			$players_beat = count($scores) - ($i + 1);
			for($j = $i + 1; $j < count($scores); $j++) {
				if($scores[$i]['playoff_winner'] != 'Yes' && $scores[$i]['total'] == $scores[$j]['total']) {
					$players_beat -= 1;
				} else {
					break;
				}
			}

			$points += $points_per_player_beat * $players_beat;

			if($last_playoff_winner != 'Yes' && $scores[$i]['total'] == $last_score) {
				$rank = $last_rank;
			} else {
				$rank = $i + 1;
			}

			if($rank == 1) {
				$points += $points_division_win;
			}

			update_post_meta($scores[$i]['score_id'], 'points', $points);
			update_post_meta($scores[$i]['score_id'], 'rank', $rank);

			$last_playoff_winner = $scores[$i]['playoff_winner'];
			$last_score = $scores[$i]['total'];
			$last_rank = $rank;
		}
	}

	function init() {
		add_action('admin_enqueue_scripts', array($this, 'adminEnqueueScripts'));
		add_action( 'admin_menu', array($this, 'adminMenu') );
		$this->capability = $this->key_ . 'edit_tournaments';

		$capabilities =  array(
			'edit_post'          => $this->key_ . 'edit_tournaments', 
			'read_post'          => $this->key_ . 'edit_tournaments', 
			'delete_posts'        => $this->key_ . 'edit_tournaments',
			'edit_posts'         => $this->key_ . 'edit_tournaments',
			'edit_others_posts'  => $this->key_ . 'edit_tournaments',
			'publish_posts'      => $this->key_ . 'edit_tournaments',
			'read_private_posts' => $this->key_ . 'edit_tournaments',
			'create_posts'       => $this->key_ . 'edit_tournaments',

			'delete_post'        => $this->key_ . 'edit_tournaments',
		);

		/*
		// Seasons
		$labels = array(
			'name'               => __( 'Seasons' ),
			'singular_name'      => __( 'Season' ),
			'add_new'            => __( 'Add New' ),
			'add_new_item'       => __( 'Add New Season' ),
			'edit_item'          => __( 'Edit Season' ),
			'new_item'           => __( 'New Season' ),
			'all_items'          => __( 'Seasons' ),
			'view_item'          => __( 'View Season' ),
			'search_items'       => __( 'Search Seasons' ),
			'not_found'          => __( 'No seasons found' ),
			'not_found_in_trash' => __( 'No seasons found in the Trash' ), 
			'parent_item_colon'  => '',
			'menu_name'          => 'Seasons'
		);
		register_taxonomy(
			$this->key_ . 'seasons',
			$this->key_ . 'leagues',
			array(
				'labels' => $labels,
				'show_ui' => true,
				'show_in_menu' => true,
				'show_tagcloud' => false,
				'hierarchical' => true
			)
		);
		// Add taxonomy to custom menu
		// From: https://wordpress.stackexchange.com/a/195805
		add_submenu_page($this->parseToKey('Disc Golf'), 'Seasons', 'Seasons', 'edit_posts', 'edit-tags.php?taxonomy=disc_golf_seasons&post_type=disc_golf_leagues',false );

		// Highlight submenu
		// From: https://wordpress.stackexchange.com/a/160235
		add_filter( 'parent_file', array($this, 'highlightSeasonMenu'));
		 */

		// Seasons
		$capabilities =  array(
			'edit_post'          => 'install_plugins', 
			'read_post'          => 'install_plugins', 
			'delete_posts'        => 'install_plugins',
			'edit_posts'         => 'install_plugins',
			'edit_others_posts'  => 'install_plugins',
			'publish_posts'      => 'install_plugins',
			'read_private_posts' => 'install_plugins',
			'create_posts'       => 'install_plugins',

			'delete_post'        => 'install_plugins',
		);

		$labels = array(
			'name'               => __( 'Seasons' ),
			'singular_name'      => __( 'Season' ),
			'add_new'            => __( 'Add New' ),
			'add_new_item'       => __( 'Add New Season' ),
			'edit_item'          => __( 'Edit Season' ),
			'new_item'           => __( 'New Season' ),
			'all_items'          => __( 'Seasons' ),
			'view_item'          => __( 'View Season' ),
			'search_items'       => __( 'Search Seasons' ),
			'not_found'          => __( 'No seasons found' ),
			'not_found_in_trash' => __( 'No seasons found in the Trash' ), 
			'parent_item_colon'  => '',
			'menu_name'          => 'Seasons'
		);
		$args = array(
			'labels'        => $labels,
			'description'   => 'A list of Seasons.',
			'menu_position' => 5,
			'supports'      => array( 'title'),
			'public' => false,
			'show_ui' => true,
			'show_in_menu' => $this->parseToKey('Disc Golf'),
			'capabilities' => $capabilities,
			'hierarchical' => true,
			'register_meta_box_cb' => array($this, 'seasonsMeta')
		);
		register_post_type( $this->key_ . 'seasons', $args );

		add_filter('manage_edit-' . $this->key_ . 'seasons' . '_columns', array($this, 'seasonsColumns')) ;
		add_action('manage_' . $this->key_ . 'seasons' . '_posts_custom_column', array($this, 'seasonsColumnContent'), 10, 2 );

		// Highlight submenu
		// From: https://wordpress.stackexchange.com/a/160235
		//add_filter( 'parent_file', array($this, 'highlightLeaguesMenu'));

		//http://tempq.com/wp-admin/
		// Inspired by: https://wordpress.stackexchange.com/a/195805
		// Inspired by: https://wordpress.org/support/topic/user-cannot-add-new-for-custom-post-type/
		// Inspired by: https://wordpress.stackexchange.com/a/203156
		//add_submenu_page('edit.php?post_type=' . $this->key_ . 'leagues', 'Hidden', 'Hidden', $this->key_ . 'edit_tournaments', 'post-new.php?post_type=' . $this->key_ . 'leagues', false);




		// Leagues
		$capabilities =  array(
			'edit_post'          => $this->key_ . 'edit_tournaments', 
			'read_post'          => $this->key_ . 'edit_tournaments', 
			'delete_posts'        => $this->key_ . 'edit_tournaments',
			'edit_posts'         => $this->key_ . 'edit_tournaments',
			'edit_others_posts'  => $this->key_ . 'edit_tournaments',
			'publish_posts'      => $this->key_ . 'edit_tournaments',
			'read_private_posts' => $this->key_ . 'edit_tournaments',
			'create_posts'       => $this->key_ . 'edit_tournaments',

			'delete_post'        => $this->key_ . 'edit_tournaments',
		);
		$labels = array(
			'name'               => __( 'Leagues' ),
			'singular_name'      => __( 'League' ),
			'add_new'            => __( 'Add New' ),
			'add_new_item'       => __( 'Add New League' ),
			'edit_item'          => __( 'Edit League' ),
			'new_item'           => __( 'New League' ),
			'all_items'          => __( 'Leagues' ),
			'view_item'          => __( 'View League' ),
			'search_items'       => __( 'Search Leagues' ),
			'not_found'          => __( 'No leagues found' ),
			'not_found_in_trash' => __( 'No leagues found in the Trash' ), 
			'parent_item_colon'  => '',
			'menu_name'          => 'Leagues'
		);
		$args = array(
			'labels'        => $labels,
			'description'   => 'A list of Leagues.',
			'menu_position' => 5,
			'supports'      => array( 'title', 'editor'),
			'public' => false,
			'show_ui' => true,
			'show_in_menu' => $this->parseToKey('Disc Golf'),
			'capabilities' => $capabilities,
			'hierarchical' => true,
			'register_meta_box_cb' => array($this, 'leaguesMeta')
		);
		register_post_type( $this->key_ . 'leagues', $args );

		add_filter('manage_edit-' . $this->key_ . 'leagues' . '_columns', array($this, 'leagueColumns')) ;
		add_action('manage_' . $this->key_ . 'leagues' . '_posts_custom_column', array($this, 'leagueColumnContent'), 10, 2 );


		// Add taxonomy to custom menu
		// Inspired by: https://wordpress.stackexchange.com/a/195805
		// Inspired by: https://wordpress.org/support/topic/user-cannot-add-new-for-custom-post-type/
		//add_submenu_page($this->parseToKey('Disc Golf'), 'New League', 'New League', $this->key_ . 'edit_tournaments', 'post-new.php?post_type=disc_golf_leagues',false );

		// Highlight submenu
		// From: https://wordpress.stackexchange.com/a/160235
		//add_filter( 'parent_file', array($this, 'highlightLeaguesMenu'));

		//http://tempq.com/wp-admin/
		// Inspired by: https://wordpress.stackexchange.com/a/195805
		// Inspired by: https://wordpress.org/support/topic/user-cannot-add-new-for-custom-post-type/
		// Inspired by: https://wordpress.stackexchange.com/a/203156
		//add_submenu_page('edit.php?post_type=' . $this->key_ . 'leagues', 'Hidden', 'Hidden', $this->key_ . 'edit_tournaments', 'post-new.php?post_type=' . $this->key_ . 'leagues', false);

		// Highlight submenu
		// From: https://wordpress.stackexchange.com/a/160235
		//add_filter( 'parent_file', array($this, 'highlightMenu'));
		//add_filter( 'submenu_file', array($this, 'highlightMenu2'), 10, 2);

		// Events
		$capabilities =  array(
			'edit_post'          => $this->key_ . 'edit_tourn', 
			'delete_posts'        => $this->key_ . 'edit_tournaments',
			'edit_posts'         => $this->key_ . 'edit_tournaments',
			'edit_published_posts' => $this->key_ . 'edit_tournaments',
			'edit_others_posts'  => 'install_plugins',
			'publish_posts'      => $this->key_ . 'edit_tournaments',
			'read_private_posts' => 'install_plugins',
			'create_posts'       => $this->key_ . 'edit_tournaments',

			'delete_post'        => $this->key_ . 'edit_tournaments',
			'delete_others_posts' => 'install_plugins',
		);
		$labels = array(
			'name'               => __( 'Events' ),
			'singular_name'      => __( 'Event' ),
			'add_new'            => __( 'Add New' ),
			'add_new_item'       => __( 'Add New Event' ),
			'edit_item'          => __( 'Edit Event' ),
			'new_item'           => __( 'New Event' ),
			'all_items'          => __( 'Events' ),
			'view_item'          => __( 'View Event' ),
			'search_items'       => __( 'Search Events' ),
			'not_found'          => __( 'No events found' ),
			'not_found_in_trash' => __( 'No events found in the Trash' ), 
			'parent_item_colon'  => '',
			'menu_name'          => 'Events'
		);
		$args = array(
			'labels'        => $labels,
			'description'   => 'A list of Events.',
			'menu_position' => 5,
			'supports'      => array( 'title'),
			'public' => false,
			'show_ui' => true,
			'show_in_menu' => $this->parseToKey('Disc Golf'),
			'capabilities' => $capabilities,
			'map_meta_cap' => false,
			'hierarchical' => true,
			'register_meta_box_cb' => array($this, 'eventsMeta')
		);
		register_post_type( $this->key_ . 'events', $args );

		add_filter('manage_edit-' . $this->key_ . 'events' . '_columns', array($this, 'eventsColumns')) ;
		add_action('manage_' . $this->key_ . 'events' . '_posts_custom_column', array($this, 'eventsColumnContent'), 10, 2 );


		// Players
		$capabilities =  array(
			'edit_post'          => $this->key_ . 'edit_tournaments', 
			'read_post'          => $this->key_ . 'edit_tournaments', 
			'delete_posts'        => $this->key_ . 'edit_tournaments',
			'edit_posts'         => $this->key_ . 'edit_tournaments',
			'edit_others_posts'  => $this->key_ . 'edit_tournaments',
			'publish_posts'      => $this->key_ . 'edit_tournaments',
			'read_private_posts' => $this->key_ . 'edit_tournaments',
			'create_posts'       => $this->key_ . 'edit_tournaments',

			'delete_post'        => $this->key_ . 'edit_tournaments',
		);
		$labels = array(
			'name'               => __( 'Players' ),
			'singular_name'      => __( 'Player' ),
			'add_new'            => __( 'Add New' ),
			'add_new_item'       => __( 'Add New Player' ),
			'edit_item'          => __( 'Edit Player' ),
			'new_item'           => __( 'New Player' ),
			'all_items'          => __( 'Players' ),
			'view_item'          => __( 'View Player' ),
			'search_items'       => __( 'Search Players' ),
			'not_found'          => __( 'No players found' ),
			'not_found_in_trash' => __( 'No players found in the Trash' ), 
			'parent_item_colon'  => '',
			'menu_name'          => 'Players'
		);
		$args = array(
			'labels'        => $labels,
			'description'   => 'A list of Players.',
			'menu_position' => 5,
			'supports'      => array(''),
			'public' => false,
			'show_ui' => true,
			'show_in_menu' => $this->parseToKey('Disc Golf'),
			'capabilities' => $capabilities,
			'hierarchical' => true,
			'register_meta_box_cb' => array($this, 'playersMeta')
		);
		register_post_type( $this->key_ . 'players', $args );

		add_filter('manage_edit-' . $this->key_ . 'players' . '_columns', array($this, 'playersColumns')) ;
		add_action('manage_' . $this->key_ . 'players' . '_posts_custom_column', array($this, 'playersColumnContent'), 10, 2 );


		// Divisions
		$capabilities =  array(
			'edit_post'          => 'install_plugins', 
			'read_post'          => 'install_plugins', 
			'delete_posts'        => 'install_plugins',
			'edit_posts'         => 'install_plugins',
			'edit_others_posts'  => 'install_plugins',
			'publish_posts'      => 'install_plugins',
			'read_private_posts' => 'install_plugins',
			'create_posts'       => 'install_plugins',

			'delete_post'        => 'install_plugins',
		);
		$labels = array(
			'name'               => __( 'Divisions' ),
			'singular_name'      => __( 'Division' ),
			'add_new'            => __( 'Add New' ),
			'add_new_item'       => __( 'Add New Division' ),
			'edit_item'          => __( 'Edit Division' ),
			'new_item'           => __( 'New Division' ),
			'all_items'          => __( 'Divisions' ),
			'view_item'          => __( 'View Division' ),
			'search_items'       => __( 'Search Divisions' ),
			'not_found'          => __( 'No Divisions found' ),
			'not_found_in_trash' => __( 'No Divisions found in the Trash' ), 
			'parent_item_colon'  => '',
			'menu_name'          => 'Divisions'
		);
		$args = array(
			'labels'        => $labels,
			'description'   => 'A list of Divisions.',
			'menu_position' => 5,
			'supports'      => array('title'),
			'public' => false,
			'show_ui' => true,
			'show_in_menu' => $this->parseToKey('Disc Golf'),
			'capabilities' => $capabilities,
			'hierarchical' => true,
			'register_meta_box_cb' => array($this, 'divisionsMeta')
		);
		register_post_type( $this->key_ . 'divisions', $args );

		add_filter('manage_edit-' . $this->key_ . 'divisions' . '_columns', array($this, 'divisionsColumns')) ;
		add_action('manage_' . $this->key_ . 'divisions' . '_posts_custom_column', array($this, 'divisionsColumnContent'), 10, 2 );


		// Scores
		$capabilities =  array(
			'edit_post'          => $this->key_ . 'edit_tourn', 
			'delete_posts'        => $this->key_ . 'edit_tournaments',
			'edit_posts'         => $this->key_ . 'edit_tournaments',
			'edit_published_posts' => $this->key_ . 'edit_tournaments',
			'edit_others_posts'  => 'install_plugins',
			'publish_posts'      => $this->key_ . 'edit_tournaments',
			'read_private_posts' => 'install_plugins',
			'create_posts'       => $this->key_ . 'edit_tournaments',

			'delete_post'        => $this->key_ . 'edit_tournaments',
			'delete_others_posts' => 'install_plugins',
		);
		$labels = array(
			'name'               => __( 'Scores' ),
			'singular_name'      => __( 'Score' ),
			'add_new'            => __( 'Add New' ),
			'add_new_item'       => __( 'Add New Score' ),
			'edit_item'          => __( 'Edit Score' ),
			'new_item'           => __( 'New Score' ),
			'all_items'          => __( 'Scores' ),
			'view_item'          => __( 'View Score' ),
			'search_items'       => __( 'Search Scores' ),
			'not_found'          => __( 'No Scores found' ),
			'not_found_in_trash' => __( 'No Scores found in the Trash' ), 
			'parent_item_colon'  => '',
			'menu_name'          => 'Scores'
		);
		$args = array(
			'labels'        => $labels,
			'description'   => 'A list of Scores.',
			'menu_position' => 5,
			'supports'      => array(''),
			'public' => false,
			'show_ui' => true,
			'show_in_menu' => $this->parseToKey('Disc Golf'),
			'capabilities' => $capabilities,
			'register_meta_box_cb' => array($this, 'scoresMeta')
		);
		register_post_type( $this->key_ . 'scores', $args );

		// Columns to display
		add_filter('manage_edit-' . $this->key_ . 'scores' . '_columns', array($this, 'scoresColumns')) ;

		// Column content
		add_action('manage_' . $this->key_ . 'scores' . '_posts_custom_column', array($this, 'scoresColumnContent'), 10, 2 );

		// Sort columns
		add_filter( 'manage_edit-' . $this->key_ . 'scores' . '_sortable_columns', array($this, 'scoresColumnSort'));
		add_filter( 'manage_edit-' . $this->key_ . 'events' . '_sortable_columns', array($this, 'eventsColumnSort'));
		//add_action( 'pre_get_posts', array($this, 'preGetPosts'));
		add_filter( 'posts_clauses', array($this, 'postsClauses'), 10, 2);

		// Handle the title field for custom post types
		add_filter('the_title', array($this, 'theTitle'), 10, 2 );
		add_filter('pre_get_document_title', array($this, 'preGetDocumentTitle'), 10);
		// From: https://wordpress.stackexchange.com/a/14982
		add_filter('post_row_actions', array($this, 'postRowActions'), 10, 2);
		add_filter('page_row_actions', array($this, 'postRowActions'), 10, 2);

		// Handle saving
		add_action('save_post_' . $this->key_ . 'seasons', array($this, 'seasonsSave'));
		add_action('save_post_' . $this->key_ . 'events', array($this, 'eventsSave'));
		add_action('save_post_' . $this->key_ . 'leagues', array($this, 'leaguesSave'));
		add_action('save_post_' . $this->key_ . 'players', array($this, 'playersSave'));
		add_action('save_post_' . $this->key_ . 'scores', array($this, 'scoresSave'));

		// Redirect after save
		add_filter('redirect_post_location', array($this, 'redirectSave'));

		// Add custom filter
		add_action('restrict_manage_posts', array($this, 'filterPosts'));

		// Handle custom filter
		add_filter('parse_query', array($this, 'parsePosts'));

		// Remove filters as needed
		add_action('admin_init', array($this, 'adminInit'));

		// Add CSS class
		add_filter( 'admin_body_class', array($this, 'adminBodyClass'));

		add_filter( 'map_meta_cap', array($this, 'mapMetaCap'), 10, 4 );

		$this->adminAjax('import'); // Call ajaxAdminUnique()
		$this->adminAjax('settings');

		$this->shortcode('disc_golf_overview');
		$this->shortcode('disc_golf_player');
		$this->shortcode('disc_golf_division');
		$this->shortcode('disc_golf_event');
		$this->shortcode('disc_golf_future_events');

		//// BOF DEBUG
		//echo current_user_can('edit_post', 375);
		//die;

		//$current_user = wp_get_current_user();
		//print_r($current_user->allcaps);
		//echo $current_user->has_cap('edit_post', 22);
		//echo $current_user->has_cap('edit_post', 258);
		//echo $current_user->has_cap('edit_post', 375);
		//die;
	}

	/* Base Code */

	/*
		function actionAdmin() {
			$message = '';

			$message = '|' . $this->val('_wpnonce', $args['action'], 'nonce');

			for($i = 0; $i < count($args['names']); $i++) {
				$message .= '|' . $this->val($args['names'][$i], $args['titles'][$i], $args['validations'][$i]);
			}

			$message = $this->parseMessage($message);

			if(!$message) {
				for($i = 0; $i < count($args['names']); $i++) {
					$this->optionSet($args['names'][$i]);
				}
			}

			if(!$message) {
				$output = array();
				$output['status'] = 'success';
				echo json_encode($output);
			} else {
				$output = array();
				$output['status'] = 'error';
				$output['message'] = $message;
				http_response_code(400);
				echo json_encode($output);
			}
			die;
		}
		function viewAdmin() {
			$data = array();
			$data['title'] = $args['title'];
			$data['action'] = $args['action'];
			$data['titles'] = $args['titles'];
			$data['names'] = $args['names'];
			$data['elements'] = $args['elements'];
			$data['values'] = $args['values'];
			$data['descriptions'] = $args['descriptions'];
			$data['validations'] = $args['validations'];
			echo $this->show('admin', $data);
		}
	 */

	/*
	 * Menu options (https://codex.wordpress.org/Administration_Menus):
	 * For Dashboard: add_submenu_page('index.php',...)
	 * For Posts: add_submenu_page('edit.php',...)
	 * For Media: add_submenu_page('upload.php',...)
	 * For Pages: add_submenu_page('edit.php?post_type=page',...)
	 * For Comments: add_submenu_page('edit-comments.php',...)
	 * For Custom Post Types: add_submenu_page('edit.php?post_type=your_post_type',...)
	 * For Appearance: add_submenu_page('themes.php',...)
	 * For Plugins: add_submenu_page('plugins.php',...)
	 * For Users: add_submenu_page('users.php',...)
	 * For Tools: add_submenu_page('tools.php',...)
	 * For Settings: add_submenu_page('options-general.php',...)
	 */

	function __construct() {
		define(strtoupper($this->key) . '_BASE', dirname(__FILE__) . DIRECTORY_SEPARATOR);

		add_action('init', array($this, 'init'), 9);
		add_action('init', array($this, 'defaultInit'), 9);
		add_action('init', array($this, 'flashProcess'));

		register_activation_hook( __FILE__, array($this, 'baseActivation'));
		register_deactivation_hook( __FILE__, array($this, 'baseDeactivation'));
	}

	function adminAjax($input) {
		$args = array();
		$args['key'] = $input;

		$args['ajax'] = function() {
			$output = array();
			$output['status'] = 'error';
			$output['message'] = 'There was a problem processing the request. The ajax call does not appear to exist.';
			http_response_code(400);
			echo json_encode($output);
		};

		if(is_callable(array($this, 'actionAdmin' . ucfirst($this->parseToCamel($args['key']))))) {
			add_action('wp_ajax_' . $this->key_ . $args['key'], array($this,  'actionAdmin' . ucfirst($this->parseToCamel($args['key'])))); 
		} else {
			add_action('wp_ajax_' . $this->key_ . $args['key'], $args['ajax']); 
		}
	}

	function adminPage() {
		$arguments = func_get_args();
		$elements = array('text', 'textarea');
		$start = 1;

		$args = array();
		$args['title'] = $arguments[0];

		// Check if second argument is key or the start of elements
		if(!in_array($arguments[1], $elements)) {
			$args['key'] = $arguments[1];
			$start = 2;
		} else {
			$args['key'] = $arguments[0];
		}
		$args['key'] = $this->parseToKey($args['key']);
		$args['key'] = preg_replace('/^' . $this->key_ . '/', '', $args['key']);

		if($this->key == $args['key'] || $args['key'] == '') {
			$args['key'] = '';
		} else {
			$args['key'] = '_' . $args['key'];
		}


		$args['capability'] = $this->capability;
		$args['menu'] = $this->menu;
		$args['action'] = $this->key . $args['key'];
		$args['titles'] = array();
		$args['names'] = array();
		$args['elements'] = array();
		$args['values'] = array();
		$args['descriptions'] = array();
		$args['validations'] = array();

		for($i = $start; $i < count($arguments); $i++) {
			array_push($args['elements'], $arguments[$i]);

			$i++;
			array_push($args['titles'], $arguments[$i]);
			if(isset($arguments[$i + 1]) && !in_array($arguments[$i + 1], $elements)) {
				$i++;
				array_push($args['names'], $arguments[$i]);
				array_push($args['values'], $this->optionGet($this->key . '_' . $arguments[$i]));

				if(isset($arguments[$i + 1]) && !in_array($elements, $arguments[$i + 1])) {
					$i++;
					array_push($args['descriptions'], $arguments[$i]);

					if(isset($arguments[$i + 1]) && !in_array($elements, $arguments[$i + 1])) {
						$i++;
						array_push($args['validations'], $arguments[$i]);
					} else {
						array_push($args['validations'], 'required');
					}
				} else {
					array_push($args['descriptions'], '');
					array_push($args['validations'], 'required');
				}
			} else {
				array_push($args['names'], $this->parseToKey($arguments[$i]));

				array_push($args['values'], $this->optionGet($this->key . '_' . $this->parseToKey($arguments[$i])));
				array_push($args['descriptions'], '');
				array_push($args['validations'], 'required');
			}
		}

		$args['admin_enqueue_scripts'] = function($hook_suffix) use($args) {
			if(strpos($hook_suffix, '_page_') !== false) {
				$slug = explode('_page_', $hook_suffix)[1];
				if($slug == $this->key . $args['key']) {
					wp_enqueue_style($this->key . $args['key'] . '_main', plugins_url('css/admin.css', __FILE__));
					wp_enqueue_script($this->key . $args['key'] . '_main', plugins_url('js/admin.js', __FILE__), array('jquery'));

					if(is_file(plugin_dir_path(__FILE__) . 'css/admin_' . $args['key'] . '.css')) {
						wp_enqueue_style($this->key . $args['key'] . '_secondary', plugins_url('css/admin_' . $args['key'] . '.css', __FILE__));
					}

					if(is_file(plugin_dir_path(__FILE__) . 'js/admin_' . $args['key'] . '.js')) {
						wp_enqueue_script($this->key . $args['key'] . '_secondary', plugins_url('js/admin_' . $args['key'] . '.js', __FILE__));
					}
				}
			}
		};

		$args['add_menu_page'] = function() use($args) {
			$data = array();
			$data['title'] = $args['title'];
			$data['action'] = $args['action'];
			$data['titles'] = $args['titles'];
			$data['names'] = $args['names'];
			$data['elements'] = $args['elements'];
			$data['values'] = $args['values'];
			$data['descriptions'] = $args['descriptions'];
			$data['validations'] = $args['validations'];
			if(is_file(plugin_dir_path(__FILE__) . 'views/admin_' . $args['key'] . '.php')) {
				echo $this->show('admin_' . $args['key'], $data);
			} else {
				echo $this->show('admin', $data);
			}
		};

		if($args['menu'] === null) {
			$args['admin_menu'] = function() use($args) {
				// Does view method exist
				if(is_callable(array($this, 'viewAdmin' . ucfirst($this->parseToCamel($args['key']))))) {
					add_menu_page($args['title'], $args['title'], $args['capability'], $this->key . $args['key'], array($this,  'viewAdmin' . ucfirst($this->parseToCamel($args['key']))));
					add_submenu_page($this->key . $args['key'], $args['title'], $args['title'], $args['capability'], $this->key . $args['key'], array($this,  'viewAdmin' . ucfirst($this->parseToCamel($args['key']))));
				} else {
					add_menu_page($args['title'], $args['title'], $args['capability'], $this->key . $args['key'], $args['add_menu_page']);
					add_submenu_page($this->key . $args['key'], $args['title'], $args['title'], $args['capability'], $this->key . $args['key'], $args['add_menu_page']);
				}
			};
			$this->menu = $this->key . $args['key'];
		} else {
			$args['admin_menu'] = function() use($args) {
				// Does view method exist
				if(is_callable(array($this, 'viewAdmin' . ucfirst($this->parseToCamel($args['key']))))) {
					add_submenu_page($args['menu'], $args['title'], $args['title'], $args['capability'], $this->key . $args['key'], array($this,  'viewAdmin' . ucfirst($this->parseToCamel($args['key']))));
				} else {
					add_submenu_page($args['menu'], $args['title'], $args['title'], $args['capability'], $this->key . $args['key'], $args['add_menu_page']);
				}
			};
		}

		$args['ajax'] = function() use($args) {
			$message = '';

			$message = '|' . $this->val('_wpnonce', $args['action'], 'nonce');

			for($i = 0; $i < count($args['names']); $i++) {
				$message .= '|' . $this->val($args['names'][$i], $args['titles'][$i], $args['validations'][$i]);
			}

			$message = $this->parseMessage($message);

			if(!$message) {
				for($i = 0; $i < count($args['names']); $i++) {
					$this->optionSet($args['names'][$i]);
				}
			}

			if(!$message) {
				$output = array();
				$output['status'] = 'success';
				echo json_encode($output);
			} else {
				$output = array();
				$output['status'] = 'error';
				$output['message'] = $message;
				http_response_code(400);
				echo json_encode($output);
			}
			die;
		};

		add_action('admin_enqueue_scripts', $args['admin_enqueue_scripts']);
		add_action('admin_menu', $args['admin_menu']);

		if(is_callable(array($this, 'actionAdmin' . ucfirst($this->parseToCamel($args['key']))))) {
			add_action('wp_ajax_' . $this->key . $args['key'], array($this,  'actionAdmin' . ucfirst($this->parseToCamel($args['key'])))); 
		} else {
			add_action('wp_ajax_' . $this->key . $args['key'], $args['ajax']); 
		}
	}


	function actionAdminSubmit($data) {
		$message = '';

		$message = '|' . $this->val('_wpnonce', 'admin_submit', 'nonce');

		$message = $this->parseMessage($message);

		if(!$message) {
			$this->optionSet('name');
		}

		if(!$message) {
			$output = array();
			$output['status'] = 'success';
			echo json_encode($output);
		} else {
			$output = array();
			$output['status'] = 'error';
			$output['message'] = $message;
			http_response_code(400);
			echo json_encode($output);
		}
		die;
	}

	function defaultInit() {
		//add_action('template_redirect', array($this, 'flashProcess'), 5);
		add_action('admin_notices', array($this, 'notifications'));
	}


	function flashAdd($message, $type) {
		if(isset($this->flash[$type])) {
			$this->flash[$type] = $this->flash[$type] . ' ' . $message;
		} else {
			$this->flash[$type] = $message;
		}
	}

	function flashProcess() {
		if(isset($_COOKIE[$this->key . '_flash'])) {
			$temp = unserialize(base64_decode($_COOKIE[$this->key . '_flash']));

			$this->success = '';
			$this->error = '';
			foreach(array_keys($temp) as $key) {
				if($key == 'success') {
					$this->success .= $temp[$key];
				} else {
					$this->error .= $temp[$key];
				}
			}
		}
		if(count($this->flash)) {
			setcookie($this->key . '_flash', base64_encode(serialize($this->flash)), 0, '/');
		} else {
			// Remove cookie
			unset($_COOKIE[$this->key . '_flash']);
			setcookie($this->key . '_flash', '', time() - 3600, '/');
		}
	}

	// $type is usually success or error
	function flashSave($message, $type) {
		$this->flashAdd($message, $type);
		setcookie($this->key . '_flash', base64_encode(serialize($this->flash)), 0, '/');
	}

	// Activate plugin
	function baseActivation() {
		global $wpdb, $current_user, $wp_roles;

		require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

		// Setup
		$migrations = maybe_unserialize(get_option($this->key . '_migrations'));
		if(!is_array($migrations)) {
			$migrations = array();
		}
		$notifications = array();

		// Migrations
		$mig = '2018-01-15';
		$notification = 'Initial setup.';
		if(!in_array($mig, $migrations)) {
			array_push($migrations, $mig);
			array_push($notifications, $notification);
		}

		// Cleanup
		update_option($this->key . '_migrations', serialize($migrations));
		update_option($this->key . '_notifications', serialize($notifications));

		if(is_callable(array($this, 'activation'))) {
			$this->activation();
		}
	}

	// Deactivate plugin
	function baseDeactivation() {
		if(is_callable(array($this, 'deactivation'))) {
			$this->deactivation();
		}
	}

	function notifications($attr) {
		$notifications = maybe_unserialize(get_option($this->key . '_notifications'));
		if(count($notifications)) {
			update_option($this->key . '_notifications', serialize(array()));
			$data = array();
			$data['message'] = 'The ' . $this->name . ' Plugin performed the following:';
			$data['notifications'] = $notifications;
			$this->show('notifications', $data);
		}
	}

	function optionGet($key) {
		$result = get_option($this->key . '_' . $key);
		if(is_string($result)) {
			$result = stripslashes($result);
		}
		return $result;
	}

	function optionSet($key, $data) {
		if(is_null($data)) {
			update_option($this->key . '_' . $key, $_POST[$key]);
		} else {
			update_option($this->key . '_' . $key, $data);
		}
	}

	function parseToCamel($input) {
		$output = $input;
		$output = strtolower($output);
		$output = str_replace(' ', '_', $output);
		$parts = explode('_', $output);
		for($i = 0; $i < count($parts); $i++) {
			if($i == 0) {
				$output = $parts[$i];
			} else {
				$output .= ucfirst($parts[$i]);
			}
		}
		return $output;
	}

	function parseToKey($input) {
		$output = $input;
		$output = strtolower($output);
		$output = str_replace(' ', '_', $output);
		return $output;
	}

	function parseMessage($message, $insert = '<br>') {
		$message = trim($message, '|');
		$message = preg_replace('/\|+/', '|', $message);
		$messages = explode('|', $message);
		$message = implode($insert, $messages);
		return $message;
	}

	function shortcode($name) {
		$args = array();

		$args['name'] = $name;

		$args['shortcode'] = function() use($args) {
			echo '<p>The shortcode [' . $args['name'] . '] needs to be created.</p>';
		};

		$args['wp_enqueue_scripts'] = function() use($args) {
			global $post;
			if(is_a( $post, 'WP_Post') && has_shortcode($post->post_content, $args['name']) ) {
				if(is_file(plugin_dir_path(__FILE__) . 'css/shortcode_' . $args['name'] . '.css')) {
					wp_enqueue_style($this->key . '_shortcode_' . $args['name'], plugins_url('css/shortcode_' . $args['name'] . '.css', __FILE__));
				}

				if(is_file(plugin_dir_path(__FILE__) . 'js/shortcode_' . $args['name'] . '.js')) {
					wp_enqueue_script($this->key . '_shortcode_' . $args['name'], plugins_url('js/shortcode_' . $args['name'] . '.js', __FILE__), array('jquery'));
				}
			}
		};

		if($name != '' && is_callable(array($this, 'shortcode' . ucfirst($this->parseToCamel($name))))) {
			add_shortcode($name, array($this,  'shortcode' . ucfirst($this->parseToCamel($name))), 10);

			add_action('wp_enqueue_scripts', $args['wp_enqueue_scripts']);
		} else {
			add_shortcode($name, $args['shortcode'], 10);
		}
	}

	/*
	 * Outputs view files. Variables passed in via $data become accessible in the view.
	 * For example, $data['user_id'] becomes $user_id in the view file.
	 */
	function show($view, $data = null) {
		// Extract the data variable so that 
		// $data['example'] can be called by
		// $example_raw
		// $example (or escaped by htmlspecialchars)
		if(is_array($data)) {
			foreach($data as $key => $value) {
				if(is_string($value)) {
					${$key . '_raw'} = $value;
					${$key} = $value;
					// Handle escaping in the views with Wordpress functions
					//${$key} = htmlspecialchars($value, ENT_QUOTES);
				} else {
					${$key . '_raw'} = $value;
					${$key} = $value;
				}
			}
		}
		$success = $this->success;
		$error = $this->error;

		ob_start();
		include 'views' . DIRECTORY_SEPARATOR . $view . '.php';
		$output = ob_get_clean();

		return $output;
	}

	function val($item, $name, $type, $message) {
		if(isset($_POST[$item])) {
			$item = $_POST[$item];
		}

		if($type == 'nonce') {
			return $this->valNonce($item, $name, $message);
		} elseif($type == 'required') {
			return $this->valRequired($item, $name, $message);
		} else {
			return 'There was a problem processing the information.';
		}
	}

	function valNonce($item, $name, $message) {
		if(!wp_verify_nonce($item, $this->key . '_' . $name)) {
			if($message) {
				return sprintf($message, $name);
			} else {
				return sprintf('There was a problem processing the information.', $name);
			}
		} else {
			return '';
		}
	}

	function valRequired($item, $name, $message) {
		if(!isset($item)) {
			if($message) {
				return sprintf($message, $name);
			} else {
				return sprintf('The %s item is required.', $name);
			}
		} elseif(!is_numeric($item) && !$item) {
			if($message) {
				return sprintf($message, $name);
			} else {
				return sprintf('The %s item is required.', $name);
			}
		} else {
			return '';
		}
	}
}

new DiscGolfManager();

